# The contents of this file are automatically written by
# tools/generate_schema_wrapper.py. Do not modify directly.

from altair.utils.schemapi import SchemaBase, Undefined

import os
import json

def load_schema():
    """Load the json schema associated with this module's functions"""
    directory = os.path.dirname(__file__)
    with open(os.path.join(directory, 'vega-schema.json')) as f:
        return json.load(f)


class VegaSchema(SchemaBase):
    @classmethod
    def _default_wrapper_classes(cls):
        return VegaSchema.__subclasses__()


class Root(VegaSchema):
    """Root schema wrapper
    
    allOf(scope, Mapping(required=[]))
    """
    _schema = load_schema()
    _rootschema = _schema

    def __init__(self, autosize=Undefined, axes=Undefined, background=Undefined, config=Undefined,
                 data=Undefined, description=Undefined, encode=Undefined, height=Undefined,
                 layout=Undefined, legends=Undefined, marks=Undefined, padding=Undefined,
                 projections=Undefined, scales=Undefined, signals=Undefined, title=Undefined,
                 usermeta=Undefined, width=Undefined, **kwds):
        super(Root, self).__init__(autosize=autosize, axes=axes, background=background, config=config,
                                   data=data, description=description, encode=encode, height=height,
                                   layout=layout, legends=legends, marks=marks, padding=padding,
                                   projections=projections, scales=scales, signals=signals, title=title,
                                   usermeta=usermeta, width=width, **kwds)


class autosize(VegaSchema):
    """autosize schema wrapper
    
    oneOf(enum('pad', 'fit', 'fit-x', 'fit-y', 'none'), Mapping(required=[type]))
    """
    _schema = {'$ref': '#/defs/autosize'}
    _rootschema = Root._schema

    def __init__(self, *args, **kwds):
        super(autosize, self).__init__(*args, **kwds)


class axis(VegaSchema):
    """axis schema wrapper
    
    Mapping(required=[orient, scale])
    
    Attributes
    ----------
    orient : enum('top', 'bottom', 'left', 'right')
    
    scale : string
    
    domain : boolean
    
    encode : Mapping(required=[])
    
    format : stringOrSignal
    
    grid : boolean
    
    gridScale : string
    
    labelBound : oneOf(boolean, float)
    
    labelFlush : oneOf(boolean, float)
    
    labelFlushOffset : float
    
    labelOverlap : oneOf(boolean, enum('parity', 'greedy'))
    
    labelPadding : float
    
    labels : boolean
    
    maxExtent : oneOf(float, numberValue)
    
    minExtent : oneOf(float, numberValue)
    
    offset : oneOf(float, numberValue)
    
    position : oneOf(float, numberValue)
    
    tickCount : tickCount
    
    tickSize : float
    
    ticks : boolean
    
    title : stringOrSignal
    
    titlePadding : oneOf(float, numberValue)
    
    values : oneOf(List(Mapping(required=[])), signal)
    
    zindex : float
    
    """
    _schema = {'$ref': '#/defs/axis'}
    _rootschema = Root._schema

    def __init__(self, orient=Undefined, scale=Undefined, domain=Undefined, encode=Undefined,
                 format=Undefined, grid=Undefined, gridScale=Undefined, labelBound=Undefined,
                 labelFlush=Undefined, labelFlushOffset=Undefined, labelOverlap=Undefined,
                 labelPadding=Undefined, labels=Undefined, maxExtent=Undefined, minExtent=Undefined,
                 offset=Undefined, position=Undefined, tickCount=Undefined, tickSize=Undefined,
                 ticks=Undefined, title=Undefined, titlePadding=Undefined, values=Undefined,
                 zindex=Undefined, **kwds):
        super(axis, self).__init__(orient=orient, scale=scale, domain=domain, encode=encode,
                                   format=format, grid=grid, gridScale=gridScale, labelBound=labelBound,
                                   labelFlush=labelFlush, labelFlushOffset=labelFlushOffset,
                                   labelOverlap=labelOverlap, labelPadding=labelPadding, labels=labels,
                                   maxExtent=maxExtent, minExtent=minExtent, offset=offset,
                                   position=position, tickCount=tickCount, tickSize=tickSize,
                                   ticks=ticks, title=title, titlePadding=titlePadding, values=values,
                                   zindex=zindex, **kwds)


class background(VegaSchema):
    """background schema wrapper
    
    string
    """
    _schema = {'$ref': '#/defs/background'}
    _rootschema = Root._schema

    def __init__(self, *args):
        super(background, self).__init__(*args)


class bind(VegaSchema):
    """bind schema wrapper
    
    oneOf(Mapping(required=[input]), Mapping(required=[input, options]), 
    Mapping(required=[input]), Mapping(required=[]))
    """
    _schema = {'$ref': '#/defs/bind'}
    _rootschema = Root._schema

    def __init__(self, *args, **kwds):
        super(bind, self).__init__(*args, **kwds)


class dataFormat(VegaSchema):
    """dataFormat schema wrapper
    
    anyOf(Mapping(required=[]), Mapping(required=[]), Mapping(required=[]), 
    oneOf(Mapping(required=[]), Mapping(required=[])))
    """
    _schema = {'$ref': '#/defs/dataFormat'}
    _rootschema = Root._schema

    def __init__(self, *args, **kwds):
        super(dataFormat, self).__init__(*args, **kwds)


class data(VegaSchema):
    """data schema wrapper
    
    allOf(Mapping(required=[name]), anyOf(Mapping(required=[name]), 
    oneOf(Mapping(required=[source]), Mapping(required=[values]), Mapping(required=[url]))))
    """
    _schema = {'$ref': '#/defs/data'}
    _rootschema = Root._schema

    def __init__(self, name=Undefined, on=Undefined, transform=Undefined, **kwds):
        super(data, self).__init__(name=name, on=on, transform=transform, **kwds)


class rule(VegaSchema):
    """rule schema wrapper
    
    Mapping(required=[])
    
    Attributes
    ----------
    test : string
    
    """
    _schema = {'$ref': '#/defs/rule'}
    _rootschema = Root._schema

    def __init__(self, test=Undefined, **kwds):
        super(rule, self).__init__(test=test, **kwds)


class encodeEntry(VegaSchema):
    """encodeEntry schema wrapper
    
    Mapping(required=[])
    
    Attributes
    ----------
    align : oneOf(List(allOf(rule, allOf(stringModifiers, anyOf(oneOf(signal, 
    Mapping(required=[value]), Mapping(required=[field]), Mapping(required=[range])), 
    Mapping(required=[scale, value]), Mapping(required=[scale, band]), 
    Mapping(required=[offset]))))), allOf(stringModifiers, anyOf(oneOf(signal, 
    Mapping(required=[value]), Mapping(required=[field]), Mapping(required=[range])), 
    Mapping(required=[scale, value]), Mapping(required=[scale, band]), 
    Mapping(required=[offset]))))
    
    angle : numberValue
    
    baseline : oneOf(List(allOf(rule, allOf(stringModifiers, anyOf(oneOf(signal, 
    Mapping(required=[value]), Mapping(required=[field]), Mapping(required=[range])), 
    Mapping(required=[scale, value]), Mapping(required=[scale, band]), 
    Mapping(required=[offset]))))), allOf(stringModifiers, anyOf(oneOf(signal, 
    Mapping(required=[value]), Mapping(required=[field]), Mapping(required=[range])), 
    Mapping(required=[scale, value]), Mapping(required=[scale, band]), 
    Mapping(required=[offset]))))
    
    clip : booleanValue
    
    cursor : stringValue
    
    dir : stringValue
    
    dx : numberValue
    
    dy : numberValue
    
    ellipsis : stringValue
    
    endAngle : numberValue
    
    fill : colorValue
    
    fillOpacity : numberValue
    
    font : stringValue
    
    fontSize : numberValue
    
    fontStyle : stringValue
    
    fontWeight : fontWeightValue
    
    height : numberValue
    
    innerRadius : numberValue
    
    interpolate : stringValue
    
    limit : numberValue
    
    opacity : numberValue
    
    orient : oneOf(List(allOf(rule, allOf(stringModifiers, anyOf(oneOf(signal, 
    Mapping(required=[value]), Mapping(required=[field]), Mapping(required=[range])), 
    Mapping(required=[scale, value]), Mapping(required=[scale, band]), 
    Mapping(required=[offset]))))), allOf(stringModifiers, anyOf(oneOf(signal, 
    Mapping(required=[value]), Mapping(required=[field]), Mapping(required=[range])), 
    Mapping(required=[scale, value]), Mapping(required=[scale, band]), 
    Mapping(required=[offset]))))
    
    outerRadius : numberValue
    
    path : stringValue
    
    radius : numberValue
    
    shape : anyOf(string, stringValue)
    
    size : numberValue
    
    startAngle : numberValue
    
    stroke : colorValue
    
    strokeDash : arrayValue
    
    strokeDashOffset : numberValue
    
    strokeOpacity : numberValue
    
    strokeWidth : numberValue
    
    tension : numberValue
    
    text : stringValue
    
    theta : numberValue
    
    url : stringValue
    
    width : numberValue
    
    x : numberValue
    
    x2 : numberValue
    
    xc : numberValue
    
    y : numberValue
    
    y2 : numberValue
    
    yc : numberValue
    
    """
    _schema = {'$ref': '#/defs/encodeEntry'}
    _rootschema = Root._schema

    def __init__(self, align=Undefined, angle=Undefined, baseline=Undefined, clip=Undefined,
                 cursor=Undefined, dir=Undefined, dx=Undefined, dy=Undefined, ellipsis=Undefined,
                 endAngle=Undefined, fill=Undefined, fillOpacity=Undefined, font=Undefined,
                 fontSize=Undefined, fontStyle=Undefined, fontWeight=Undefined, height=Undefined,
                 innerRadius=Undefined, interpolate=Undefined, limit=Undefined, opacity=Undefined,
                 orient=Undefined, outerRadius=Undefined, path=Undefined, radius=Undefined,
                 shape=Undefined, size=Undefined, startAngle=Undefined, stroke=Undefined,
                 strokeDash=Undefined, strokeDashOffset=Undefined, strokeOpacity=Undefined,
                 strokeWidth=Undefined, tension=Undefined, text=Undefined, theta=Undefined,
                 url=Undefined, width=Undefined, x=Undefined, x2=Undefined, xc=Undefined, y=Undefined,
                 y2=Undefined, yc=Undefined, **kwds):
        super(encodeEntry, self).__init__(align=align, angle=angle, baseline=baseline, clip=clip,
                                          cursor=cursor, dir=dir, dx=dx, dy=dy, ellipsis=ellipsis,
                                          endAngle=endAngle, fill=fill, fillOpacity=fillOpacity,
                                          font=font, fontSize=fontSize, fontStyle=fontStyle,
                                          fontWeight=fontWeight, height=height, innerRadius=innerRadius,
                                          interpolate=interpolate, limit=limit, opacity=opacity,
                                          orient=orient, outerRadius=outerRadius, path=path,
                                          radius=radius, shape=shape, size=size, startAngle=startAngle,
                                          stroke=stroke, strokeDash=strokeDash,
                                          strokeDashOffset=strokeDashOffset,
                                          strokeOpacity=strokeOpacity, strokeWidth=strokeWidth,
                                          tension=tension, text=text, theta=theta, url=url, width=width,
                                          x=x, x2=x2, xc=xc, y=y, y2=y2, yc=yc, **kwds)


class encode(VegaSchema):
    """encode schema wrapper
    
    Mapping(required=[])
    """
    _schema = {'$ref': '#/defs/encode'}
    _rootschema = Root._schema

    def __init__(self, **kwds):
        super(encode, self).__init__(**kwds)


class layout(VegaSchema):
    """layout schema wrapper
    
    oneOf(Mapping(required=[]), signal)
    """
    _schema = {'$ref': '#/defs/layout'}
    _rootschema = Root._schema

    def __init__(self, *args, **kwds):
        super(layout, self).__init__(*args, **kwds)


class guideEncode(VegaSchema):
    """guideEncode schema wrapper
    
    Mapping(required=[])
    
    Attributes
    ----------
    interactive : boolean
    
    name : string
    
    style : style
    
    """
    _schema = {'$ref': '#/defs/guideEncode'}
    _rootschema = Root._schema

    def __init__(self, interactive=Undefined, name=Undefined, style=Undefined, **kwds):
        super(guideEncode, self).__init__(interactive=interactive, name=name, style=style, **kwds)


class legend(VegaSchema):
    """legend schema wrapper
    
    anyOf(Mapping(required=[size]), Mapping(required=[shape]), Mapping(required=[fill]), 
    Mapping(required=[stroke]), Mapping(required=[opacity]), Mapping(required=[strokeDash]))
    
    Attributes
    ----------
    """
    _schema = {'$ref': '#/defs/legend'}
    _rootschema = Root._schema

    def __init__(self, *args, **kwds):
        super(legend, self).__init__(*args, **kwds)


class mark(VegaSchema):
    """mark schema wrapper
    
    Mapping(required=[type])
    
    Attributes
    ----------
    type : marktype
    
    clip : markclip
    
    encode : encode
    
    interactive : booleanOrSignal
    
    key : string
    
    name : string
    
    on : onMarkTrigger
    
    role : string
    
    sort : compare
    
    style : style
    
    transform : List(transformMark)
    
    """
    _schema = {'$ref': '#/defs/mark'}
    _rootschema = Root._schema

    def __init__(self, type=Undefined, clip=Undefined, encode=Undefined, interactive=Undefined,
                 key=Undefined, name=Undefined, on=Undefined, role=Undefined, sort=Undefined,
                 style=Undefined, transform=Undefined, **kwds):
        super(mark, self).__init__(type=type, clip=clip, encode=encode, interactive=interactive,
                                   key=key, name=name, on=on, role=role, sort=sort, style=style,
                                   transform=transform, **kwds)


class markGroup(VegaSchema):
    """markGroup schema wrapper
    
    allOf(Mapping(required=[type]), mark, scope, Mapping(required=[]))
    """
    _schema = {'$ref': '#/defs/markGroup'}
    _rootschema = Root._schema

    def __init__(self, type=Undefined, axes=Undefined, clip=Undefined, data=Undefined, encode=Undefined,
                 interactive=Undefined, key=Undefined, layout=Undefined, legends=Undefined,
                 marks=Undefined, name=Undefined, on=Undefined, projections=Undefined, role=Undefined,
                 scales=Undefined, signals=Undefined, sort=Undefined, style=Undefined, title=Undefined,
                 transform=Undefined, **kwds):
        super(markGroup, self).__init__(type=type, axes=axes, clip=clip, data=data, encode=encode,
                                        interactive=interactive, key=key, layout=layout,
                                        legends=legends, marks=marks, name=name, on=on,
                                        projections=projections, role=role, scales=scales,
                                        signals=signals, sort=sort, style=style, title=title,
                                        transform=transform, **kwds)


class markVisual(VegaSchema):
    """markVisual schema wrapper
    
    allOf(not Mapping(required=[]), mark, Mapping(required=[]))
    """
    _schema = {'$ref': '#/defs/markVisual'}
    _rootschema = Root._schema

    def __init__(self, type=Undefined, clip=Undefined, encode=Undefined, interactive=Undefined,
                 key=Undefined, name=Undefined, on=Undefined, role=Undefined, sort=Undefined,
                 style=Undefined, transform=Undefined, **kwds):
        super(markVisual, self).__init__(type=type, clip=clip, encode=encode, interactive=interactive,
                                         key=key, name=name, on=on, role=role, sort=sort, style=style,
                                         transform=transform, **kwds)


class listener(VegaSchema):
    """listener schema wrapper
    
    oneOf(signal, Mapping(required=[scale]), stream)
    """
    _schema = {'$ref': '#/defs/listener'}
    _rootschema = Root._schema

    def __init__(self, *args, **kwds):
        super(listener, self).__init__(*args, **kwds)


class onEvents(VegaSchema):
    """onEvents schema wrapper
    
    List(allOf(Mapping(required=[events]), oneOf(Mapping(required=[encode]), 
    Mapping(required=[update]))))
    """
    _schema = {'$ref': '#/defs/onEvents'}
    _rootschema = Root._schema

    def __init__(self, *args):
        super(onEvents, self).__init__(*args)


class onTrigger(VegaSchema):
    """onTrigger schema wrapper
    
    List(Mapping(required=[trigger]))
    """
    _schema = {'$ref': '#/defs/onTrigger'}
    _rootschema = Root._schema

    def __init__(self, *args):
        super(onTrigger, self).__init__(*args)


class onMarkTrigger(VegaSchema):
    """onMarkTrigger schema wrapper
    
    List(Mapping(required=[trigger]))
    """
    _schema = {'$ref': '#/defs/onMarkTrigger'}
    _rootschema = Root._schema

    def __init__(self, *args):
        super(onMarkTrigger, self).__init__(*args)


class padding(VegaSchema):
    """padding schema wrapper
    
    oneOf(float, Mapping(required=[]))
    """
    _schema = {'$ref': '#/defs/padding'}
    _rootschema = Root._schema

    def __init__(self, *args, **kwds):
        super(padding, self).__init__(*args, **kwds)


class projection(VegaSchema):
    """projection schema wrapper
    
    Mapping(required=[name])
    
    Attributes
    ----------
    name : string
    
    center : oneOf(signal, List(numberOrSignal))
    
    clipAngle : numberOrSignal
    
    clipExtent : oneOf(signal, List(oneOf(signal, List(numberOrSignal))))
    
    extent : oneOf(signal, List(oneOf(signal, List(numberOrSignal))))
    
    fit : oneOf(Mapping(required=[]), List(Mapping(required=[])))
    
    parallels : oneOf(signal, List(numberOrSignal))
    
    pointRadius : numberOrSignal
    
    precision : numberOrSignal
    
    rotate : oneOf(signal, List(numberOrSignal))
    
    scale : numberOrSignal
    
    size : oneOf(signal, List(numberOrSignal))
    
    translate : oneOf(signal, List(numberOrSignal))
    
    type : stringOrSignal
    
    """
    _schema = {'$ref': '#/defs/projection'}
    _rootschema = Root._schema

    def __init__(self, name=Undefined, center=Undefined, clipAngle=Undefined, clipExtent=Undefined,
                 extent=Undefined, fit=Undefined, parallels=Undefined, pointRadius=Undefined,
                 precision=Undefined, rotate=Undefined, scale=Undefined, size=Undefined,
                 translate=Undefined, type=Undefined, **kwds):
        super(projection, self).__init__(name=name, center=center, clipAngle=clipAngle,
                                         clipExtent=clipExtent, extent=extent, fit=fit,
                                         parallels=parallels, pointRadius=pointRadius,
                                         precision=precision, rotate=rotate, scale=scale, size=size,
                                         translate=translate, type=type, **kwds)


class scale(VegaSchema):
    """scale schema wrapper
    
    allOf(Mapping(required=[name]), oneOf(Mapping(required=[type]), Mapping(required=[type]), 
    Mapping(required=[type]), Mapping(required=[type, range]), Mapping(required=[type]), 
    Mapping(required=[type]), Mapping(required=[type]), not Mapping(required=[]), 
    Mapping(required=[type]), Mapping(required=[type]), Mapping(required=[type])))
    """
    _schema = {'$ref': '#/defs/scale'}
    _rootschema = Root._schema

    def __init__(self, name=Undefined, domain=Undefined, domainMax=Undefined, domainMid=Undefined,
                 domainMin=Undefined, domainRaw=Undefined, reverse=Undefined, round=Undefined,
                 type=Undefined, **kwds):
        super(scale, self).__init__(name=name, domain=domain, domainMax=domainMax, domainMid=domainMid,
                                    domainMin=domainMin, domainRaw=domainRaw, reverse=reverse,
                                    round=round, type=type, **kwds)


class scope(VegaSchema):
    """scope schema wrapper
    
    Mapping(required=[])
    
    Attributes
    ----------
    axes : List(axis)
    
    data : List(data)
    
    encode : encode
    
    layout : layout
    
    legends : List(legend)
    
    marks : List(oneOf(markGroup, markVisual))
    
    projections : List(projection)
    
    scales : List(scale)
    
    signals : List(signal)
    
    title : title
    
    """
    _schema = {'$ref': '#/defs/scope'}
    _rootschema = Root._schema

    def __init__(self, axes=Undefined, data=Undefined, encode=Undefined, layout=Undefined,
                 legends=Undefined, marks=Undefined, projections=Undefined, scales=Undefined,
                 signals=Undefined, title=Undefined, **kwds):
        super(scope, self).__init__(axes=axes, data=data, encode=encode, layout=layout, legends=legends,
                                    marks=marks, projections=projections, scales=scales,
                                    signals=signals, title=title, **kwds)


class signal(VegaSchema):
    """signal schema wrapper
    
    oneOf(signalPush, signalNew)
    """
    _schema = {'$ref': '#/defs/signal'}
    _rootschema = Root._schema

    def __init__(self, *args, **kwds):
        super(signal, self).__init__(*args, **kwds)


class signalName(VegaSchema):
    """signalName schema wrapper
    
    not Mapping(required=[])
    """
    _schema = {'$ref': '#/defs/signalName'}
    _rootschema = Root._schema

    def __init__(self, *args):
        super(signalName, self).__init__(*args)


class signalNew(VegaSchema):
    """signalNew schema wrapper
    
    Mapping(required=[name])
    
    Attributes
    ----------
    name : signalName
    
    bind : bind
    
    description : string
    
    on : onEvents
    
    react : boolean
    
    update : exprString
    
    value : Mapping(required=[])
    
    """
    _schema = {'$ref': '#/defs/signalNew'}
    _rootschema = Root._schema

    def __init__(self, name=Undefined, bind=Undefined, description=Undefined, on=Undefined,
                 react=Undefined, update=Undefined, value=Undefined, **kwds):
        super(signalNew, self).__init__(name=name, bind=bind, description=description, on=on,
                                        react=react, update=update, value=value, **kwds)


class signalPush(VegaSchema):
    """signalPush schema wrapper
    
    Mapping(required=[name, push])
    
    Attributes
    ----------
    name : signalName
    
    push : enum('outer')
    
    description : string
    
    on : onEvents
    
    """
    _schema = {'$ref': '#/defs/signalPush'}
    _rootschema = Root._schema

    def __init__(self, name=Undefined, push=Undefined, description=Undefined, on=Undefined, **kwds):
        super(signalPush, self).__init__(name=name, push=push, description=description, on=on, **kwds)


class streamParams(VegaSchema):
    """streamParams schema wrapper
    
    Mapping(required=[])
    
    Attributes
    ----------
    between : List(stream)
    
    consume : boolean
    
    debounce : float
    
    filter : oneOf(exprString, List(exprString))
    
    markname : string
    
    marktype : string
    
    throttle : float
    
    """
    _schema = {'$ref': '#/defs/streamParams'}
    _rootschema = Root._schema

    def __init__(self, between=Undefined, consume=Undefined, debounce=Undefined, filter=Undefined,
                 markname=Undefined, marktype=Undefined, throttle=Undefined, **kwds):
        super(streamParams, self).__init__(between=between, consume=consume, debounce=debounce,
                                           filter=filter, markname=markname, marktype=marktype,
                                           throttle=throttle, **kwds)


class streamEvents(VegaSchema):
    """streamEvents schema wrapper
    
    Mapping(required=[type])
    
    Attributes
    ----------
    type : string
    
    source : string
    
    """
    _schema = {'$ref': '#/defs/streamEvents'}
    _rootschema = Root._schema

    def __init__(self, type=Undefined, source=Undefined, **kwds):
        super(streamEvents, self).__init__(type=type, source=source, **kwds)


class stream(VegaSchema):
    """stream schema wrapper
    
    allOf(streamParams, oneOf(streamEvents, Mapping(required=[stream]), 
    Mapping(required=[merge])))
    """
    _schema = {'$ref': '#/defs/stream'}
    _rootschema = Root._schema

    def __init__(self, between=Undefined, consume=Undefined, debounce=Undefined, filter=Undefined,
                 markname=Undefined, marktype=Undefined, throttle=Undefined, **kwds):
        super(stream, self).__init__(between=between, consume=consume, debounce=debounce, filter=filter,
                                     markname=markname, marktype=marktype, throttle=throttle, **kwds)


class titleEncode(VegaSchema):
    """titleEncode schema wrapper
    
    Mapping(required=[])
    """
    _schema = {'$ref': '#/defs/titleEncode'}
    _rootschema = Root._schema

    def __init__(self, **kwds):
        super(titleEncode, self).__init__(**kwds)


class title(VegaSchema):
    """title schema wrapper
    
    oneOf(string, Mapping(required=[text]))
    """
    _schema = {'$ref': '#/defs/title'}
    _rootschema = Root._schema

    def __init__(self, *args, **kwds):
        super(title, self).__init__(*args, **kwds)


class transform(VegaSchema):
    """transform schema wrapper
    
    oneOf(aggregateTransform, binTransform, collectTransform, countpatternTransform, 
    crossTransform, densityTransform, extentTransform, filterTransform, flattenTransform, 
    foldTransform, formulaTransform, imputeTransform, joinaggregateTransform, lookupTransform, 
    pivotTransform, projectTransform, sampleTransform, sequenceTransform, windowTransform, 
    identifierTransform, linkpathTransform, pieTransform, stackTransform, contourTransform, 
    geojsonTransform, geopathTransform, geopointTransform, geoshapeTransform, 
    graticuleTransform, forceTransform, nestTransform, packTransform, partitionTransform, 
    stratifyTransform, treeTransform, treelinksTransform, treemapTransform, voronoiTransform, 
    wordcloudTransform, crossfilterTransform, resolvefilterTransform)
    """
    _schema = {'$ref': '#/defs/transform'}
    _rootschema = Root._schema

    def __init__(self, *args, **kwds):
        super(transform, self).__init__(*args, **kwds)


class transformMark(VegaSchema):
    """transformMark schema wrapper
    
    oneOf(binTransform, collectTransform, extentTransform, formulaTransform, 
    joinaggregateTransform, lookupTransform, sampleTransform, windowTransform, 
    identifierTransform, linkpathTransform, pieTransform, stackTransform, geojsonTransform, 
    geopathTransform, geopointTransform, geoshapeTransform, forceTransform, packTransform, 
    partitionTransform, stratifyTransform, treeTransform, treemapTransform, voronoiTransform, 
    wordcloudTransform, crossfilterTransform, resolvefilterTransform)
    """
    _schema = {'$ref': '#/defs/transformMark'}
    _rootschema = Root._schema

    def __init__(self, *args, **kwds):
        super(transformMark, self).__init__(*args, **kwds)


class aggregateTransform(VegaSchema):
    """aggregateTransform schema wrapper
    
    Mapping(required=[type])
    
    Attributes
    ----------
    type : enum('aggregate')
    
    cross : anyOf(boolean, signal)
    
    drop : anyOf(boolean, signal)
    
    fields : oneOf(List(oneOf(scaleField, paramField, expr, None)), signal)
    
    groupby : oneOf(List(oneOf(scaleField, paramField, expr)), signal)
    
    key : oneOf(scaleField, paramField, expr)
    
    ops : oneOf(List(anyOf(enum('values', 'count', '__count__', 'missing', 'valid', 'sum', 
    'mean', 'average', 'variance', 'variancep', 'stdev', 'stdevp', 'stderr', 'distinct', 'ci0', 
    'ci1', 'median', 'q1', 'q3', 'argmin', 'argmax', 'min', 'max'), signal)), signal)
    
    signal : string
    
    """
    _schema = {'$ref': '#/defs/aggregateTransform'}
    _rootschema = Root._schema

    def __init__(self, type=Undefined, cross=Undefined, drop=Undefined, fields=Undefined,
                 groupby=Undefined, key=Undefined, ops=Undefined, signal=Undefined, **kwds):
        super(aggregateTransform, self).__init__(type=type, cross=cross, drop=drop, fields=fields,
                                                 groupby=groupby, key=key, ops=ops, signal=signal,
                                                 **kwds)


class binTransform(VegaSchema):
    """binTransform schema wrapper
    
    Mapping(required=[type, field, extent])
    
    Attributes
    ----------
    extent : oneOf(List(anyOf(float, signal)), signal)
    
    field : oneOf(scaleField, paramField, expr)
    
    type : enum('bin')
    
    anchor : anyOf(float, signal)
    
    base : anyOf(float, signal)
    
    divide : oneOf(List(anyOf(float, signal)), signal)
    
    maxbins : anyOf(float, signal)
    
    minstep : anyOf(float, signal)
    
    name : anyOf(string, signal)
    
    nice : anyOf(boolean, signal)
    
    signal : string
    
    step : anyOf(float, signal)
    
    steps : oneOf(List(anyOf(float, signal)), signal)
    
    """
    _schema = {'$ref': '#/defs/binTransform'}
    _rootschema = Root._schema

    def __init__(self, extent=Undefined, field=Undefined, type=Undefined, anchor=Undefined,
                 base=Undefined, divide=Undefined, maxbins=Undefined, minstep=Undefined, name=Undefined,
                 nice=Undefined, signal=Undefined, step=Undefined, steps=Undefined, **kwds):
        super(binTransform, self).__init__(extent=extent, field=field, type=type, anchor=anchor,
                                           base=base, divide=divide, maxbins=maxbins, minstep=minstep,
                                           name=name, nice=nice, signal=signal, step=step, steps=steps,
                                           **kwds)


class collectTransform(VegaSchema):
    """collectTransform schema wrapper
    
    Mapping(required=[type])
    
    Attributes
    ----------
    type : enum('collect')
    
    signal : string
    
    sort : compare
    
    """
    _schema = {'$ref': '#/defs/collectTransform'}
    _rootschema = Root._schema

    def __init__(self, type=Undefined, signal=Undefined, sort=Undefined, **kwds):
        super(collectTransform, self).__init__(type=type, signal=signal, sort=sort, **kwds)


class countpatternTransform(VegaSchema):
    """countpatternTransform schema wrapper
    
    Mapping(required=[type, field])
    
    Attributes
    ----------
    field : oneOf(scaleField, paramField, expr)
    
    type : enum('countpattern')
    
    case : anyOf(enum('upper', 'lower', 'mixed'), signal)
    
    pattern : anyOf(string, signal)
    
    signal : string
    
    stopwords : anyOf(string, signal)
    
    """
    _schema = {'$ref': '#/defs/countpatternTransform'}
    _rootschema = Root._schema

    def __init__(self, field=Undefined, type=Undefined, case=Undefined, pattern=Undefined,
                 signal=Undefined, stopwords=Undefined, **kwds):
        super(countpatternTransform, self).__init__(field=field, type=type, case=case, pattern=pattern,
                                                    signal=signal, stopwords=stopwords, **kwds)


class crossTransform(VegaSchema):
    """crossTransform schema wrapper
    
    Mapping(required=[type])
    
    Attributes
    ----------
    type : enum('cross')
    
    filter : exprString
    
    signal : string
    
    """
    _schema = {'$ref': '#/defs/crossTransform'}
    _rootschema = Root._schema

    def __init__(self, type=Undefined, filter=Undefined, signal=Undefined, **kwds):
        super(crossTransform, self).__init__(type=type, filter=filter, signal=signal, **kwds)


class densityTransform(VegaSchema):
    """densityTransform schema wrapper
    
    Mapping(required=[type])
    
    Attributes
    ----------
    type : enum('density')
    
    distribution : oneOf(Mapping(required=[function]), Mapping(required=[function]), 
    Mapping(required=[function, field]), Mapping(required=[function]))
    
    extent : oneOf(List(anyOf(float, signal)), signal)
    
    method : anyOf(string, signal)
    
    signal : string
    
    steps : anyOf(float, signal)
    
    """
    _schema = {'$ref': '#/defs/densityTransform'}
    _rootschema = Root._schema

    def __init__(self, type=Undefined, distribution=Undefined, extent=Undefined, method=Undefined,
                 signal=Undefined, steps=Undefined, **kwds):
        super(densityTransform, self).__init__(type=type, distribution=distribution, extent=extent,
                                               method=method, signal=signal, steps=steps, **kwds)


class extentTransform(VegaSchema):
    """extentTransform schema wrapper
    
    Mapping(required=[type, field])
    
    Attributes
    ----------
    field : oneOf(scaleField, paramField, expr)
    
    type : enum('extent')
    
    signal : string
    
    """
    _schema = {'$ref': '#/defs/extentTransform'}
    _rootschema = Root._schema

    def __init__(self, field=Undefined, type=Undefined, signal=Undefined, **kwds):
        super(extentTransform, self).__init__(field=field, type=type, signal=signal, **kwds)


class filterTransform(VegaSchema):
    """filterTransform schema wrapper
    
    Mapping(required=[type, expr])
    
    Attributes
    ----------
    expr : exprString
    
    type : enum('filter')
    
    signal : string
    
    """
    _schema = {'$ref': '#/defs/filterTransform'}
    _rootschema = Root._schema

    def __init__(self, expr=Undefined, type=Undefined, signal=Undefined, **kwds):
        super(filterTransform, self).__init__(expr=expr, type=type, signal=signal, **kwds)


class flattenTransform(VegaSchema):
    """flattenTransform schema wrapper
    
    Mapping(required=[type, fields])
    
    Attributes
    ----------
    fields : oneOf(List(oneOf(scaleField, paramField, expr)), signal)
    
    type : enum('flatten')
    
    signal : string
    
    """
    _schema = {'$ref': '#/defs/flattenTransform'}
    _rootschema = Root._schema

    def __init__(self, fields=Undefined, type=Undefined, signal=Undefined, **kwds):
        super(flattenTransform, self).__init__(fields=fields, type=type, signal=signal, **kwds)


class foldTransform(VegaSchema):
    """foldTransform schema wrapper
    
    Mapping(required=[type, fields])
    
    Attributes
    ----------
    fields : oneOf(List(oneOf(scaleField, paramField, expr)), signal)
    
    type : enum('fold')
    
    signal : string
    
    """
    _schema = {'$ref': '#/defs/foldTransform'}
    _rootschema = Root._schema

    def __init__(self, fields=Undefined, type=Undefined, signal=Undefined, **kwds):
        super(foldTransform, self).__init__(fields=fields, type=type, signal=signal, **kwds)


class formulaTransform(VegaSchema):
    """formulaTransform schema wrapper
    
    Mapping(required=[type, expr, as])
    
    Attributes
    ----------
    expr : exprString
    
    type : enum('formula')
    
    initonly : anyOf(boolean, signal)
    
    signal : string
    
    """
    _schema = {'$ref': '#/defs/formulaTransform'}
    _rootschema = Root._schema

    def __init__(self, expr=Undefined, type=Undefined, initonly=Undefined, signal=Undefined, **kwds):
        super(formulaTransform, self).__init__(expr=expr, type=type, initonly=initonly, signal=signal,
                                               **kwds)


class imputeTransform(VegaSchema):
    """imputeTransform schema wrapper
    
    Mapping(required=[type, field, key])
    
    Attributes
    ----------
    field : oneOf(scaleField, paramField, expr)
    
    key : oneOf(scaleField, paramField, expr)
    
    type : enum('impute')
    
    groupby : oneOf(List(oneOf(scaleField, paramField, expr)), signal)
    
    keyvals : oneOf(List(Mapping(required=[])), signal)
    
    method : anyOf(enum('value', 'mean', 'median', 'max', 'min'), signal)
    
    signal : string
    
    value : Mapping(required=[])
    
    """
    _schema = {'$ref': '#/defs/imputeTransform'}
    _rootschema = Root._schema

    def __init__(self, field=Undefined, key=Undefined, type=Undefined, groupby=Undefined,
                 keyvals=Undefined, method=Undefined, signal=Undefined, value=Undefined, **kwds):
        super(imputeTransform, self).__init__(field=field, key=key, type=type, groupby=groupby,
                                              keyvals=keyvals, method=method, signal=signal,
                                              value=value, **kwds)


class joinaggregateTransform(VegaSchema):
    """joinaggregateTransform schema wrapper
    
    Mapping(required=[type])
    
    Attributes
    ----------
    type : enum('joinaggregate')
    
    fields : oneOf(List(oneOf(scaleField, paramField, expr, None)), signal)
    
    groupby : oneOf(List(oneOf(scaleField, paramField, expr)), signal)
    
    key : oneOf(scaleField, paramField, expr)
    
    ops : oneOf(List(anyOf(enum('values', 'count', '__count__', 'missing', 'valid', 'sum', 
    'mean', 'average', 'variance', 'variancep', 'stdev', 'stdevp', 'stderr', 'distinct', 'ci0', 
    'ci1', 'median', 'q1', 'q3', 'argmin', 'argmax', 'min', 'max'), signal)), signal)
    
    signal : string
    
    """
    _schema = {'$ref': '#/defs/joinaggregateTransform'}
    _rootschema = Root._schema

    def __init__(self, type=Undefined, fields=Undefined, groupby=Undefined, key=Undefined,
                 ops=Undefined, signal=Undefined, **kwds):
        super(joinaggregateTransform, self).__init__(type=type, fields=fields, groupby=groupby, key=key,
                                                     ops=ops, signal=signal, **kwds)


class lookupTransform(VegaSchema):
    """lookupTransform schema wrapper
    
    Mapping(required=[type, from, key, fields])
    
    Attributes
    ----------
    fields : oneOf(List(oneOf(scaleField, paramField, expr)), signal)
    
    key : oneOf(scaleField, paramField, expr)
    
    type : enum('lookup')
    
    default : Mapping(required=[])
    
    signal : string
    
    values : oneOf(List(oneOf(scaleField, paramField, expr)), signal)
    
    """
    _schema = {'$ref': '#/defs/lookupTransform'}
    _rootschema = Root._schema

    def __init__(self, fields=Undefined, key=Undefined, type=Undefined, default=Undefined,
                 signal=Undefined, values=Undefined, **kwds):
        super(lookupTransform, self).__init__(fields=fields, key=key, type=type, default=default,
                                              signal=signal, values=values, **kwds)


class pivotTransform(VegaSchema):
    """pivotTransform schema wrapper
    
    Mapping(required=[type, field, value])
    
    Attributes
    ----------
    field : oneOf(scaleField, paramField, expr)
    
    type : enum('pivot')
    
    value : oneOf(scaleField, paramField, expr)
    
    groupby : oneOf(List(oneOf(scaleField, paramField, expr)), signal)
    
    key : oneOf(scaleField, paramField, expr)
    
    limit : anyOf(float, signal)
    
    op : anyOf(enum('values', 'count', '__count__', 'missing', 'valid', 'sum', 'mean', 
    'average', 'variance', 'variancep', 'stdev', 'stdevp', 'stderr', 'distinct', 'ci0', 'ci1', 
    'median', 'q1', 'q3', 'argmin', 'argmax', 'min', 'max'), signal)
    
    signal : string
    
    """
    _schema = {'$ref': '#/defs/pivotTransform'}
    _rootschema = Root._schema

    def __init__(self, field=Undefined, type=Undefined, value=Undefined, groupby=Undefined,
                 key=Undefined, limit=Undefined, op=Undefined, signal=Undefined, **kwds):
        super(pivotTransform, self).__init__(field=field, type=type, value=value, groupby=groupby,
                                             key=key, limit=limit, op=op, signal=signal, **kwds)


class projectTransform(VegaSchema):
    """projectTransform schema wrapper
    
    Mapping(required=[type])
    
    Attributes
    ----------
    type : enum('project')
    
    fields : oneOf(List(oneOf(scaleField, paramField, expr)), signal)
    
    signal : string
    
    """
    _schema = {'$ref': '#/defs/projectTransform'}
    _rootschema = Root._schema

    def __init__(self, type=Undefined, fields=Undefined, signal=Undefined, **kwds):
        super(projectTransform, self).__init__(type=type, fields=fields, signal=signal, **kwds)


class sampleTransform(VegaSchema):
    """sampleTransform schema wrapper
    
    Mapping(required=[type])
    
    Attributes
    ----------
    type : enum('sample')
    
    signal : string
    
    size : anyOf(float, signal)
    
    """
    _schema = {'$ref': '#/defs/sampleTransform'}
    _rootschema = Root._schema

    def __init__(self, type=Undefined, signal=Undefined, size=Undefined, **kwds):
        super(sampleTransform, self).__init__(type=type, signal=signal, size=size, **kwds)


class sequenceTransform(VegaSchema):
    """sequenceTransform schema wrapper
    
    Mapping(required=[type, start, stop])
    
    Attributes
    ----------
    start : anyOf(float, signal)
    
    stop : anyOf(float, signal)
    
    type : enum('sequence')
    
    signal : string
    
    step : anyOf(float, signal)
    
    """
    _schema = {'$ref': '#/defs/sequenceTransform'}
    _rootschema = Root._schema

    def __init__(self, start=Undefined, stop=Undefined, type=Undefined, signal=Undefined,
                 step=Undefined, **kwds):
        super(sequenceTransform, self).__init__(start=start, stop=stop, type=type, signal=signal,
                                                step=step, **kwds)


class windowTransform(VegaSchema):
    """windowTransform schema wrapper
    
    Mapping(required=[type])
    
    Attributes
    ----------
    type : enum('window')
    
    fields : oneOf(List(oneOf(scaleField, paramField, expr, None)), signal)
    
    frame : oneOf(List(anyOf(float, signal, None)), signal)
    
    groupby : oneOf(List(oneOf(scaleField, paramField, expr)), signal)
    
    ignorePeers : anyOf(boolean, signal)
    
    ops : oneOf(List(anyOf(enum('row_number', 'rank', 'dense_rank', 'percent_rank', 'cume_dist',
     'ntile', 'lag', 'lead', 'first_value', 'last_value', 'nth_value', 'values', 'count', 
    '__count__', 'missing', 'valid', 'sum', 'mean', 'average', 'variance', 'variancep', 'stdev',
     'stdevp', 'stderr', 'distinct', 'ci0', 'ci1', 'median', 'q1', 'q3', 'argmin', 'argmax', 
    'min', 'max'), signal)), signal)
    
    params : oneOf(List(anyOf(float, signal, None)), signal)
    
    signal : string
    
    sort : compare
    
    """
    _schema = {'$ref': '#/defs/windowTransform'}
    _rootschema = Root._schema

    def __init__(self, type=Undefined, fields=Undefined, frame=Undefined, groupby=Undefined,
                 ignorePeers=Undefined, ops=Undefined, params=Undefined, signal=Undefined,
                 sort=Undefined, **kwds):
        super(windowTransform, self).__init__(type=type, fields=fields, frame=frame, groupby=groupby,
                                              ignorePeers=ignorePeers, ops=ops, params=params,
                                              signal=signal, sort=sort, **kwds)


class identifierTransform(VegaSchema):
    """identifierTransform schema wrapper
    
    Mapping(required=[type, as])
    
    Attributes
    ----------
    type : enum('identifier')
    
    signal : string
    
    """
    _schema = {'$ref': '#/defs/identifierTransform'}
    _rootschema = Root._schema

    def __init__(self, type=Undefined, signal=Undefined, **kwds):
        super(identifierTransform, self).__init__(type=type, signal=signal, **kwds)


class linkpathTransform(VegaSchema):
    """linkpathTransform schema wrapper
    
    Mapping(required=[type])
    
    Attributes
    ----------
    type : enum('linkpath')
    
    orient : anyOf(enum('horizontal', 'vertical', 'radial'), signal)
    
    shape : anyOf(enum('line', 'arc', 'curve', 'diagonal', 'orthogonal'), signal)
    
    signal : string
    
    sourceX : oneOf(scaleField, paramField, expr)
    
    sourceY : oneOf(scaleField, paramField, expr)
    
    targetX : oneOf(scaleField, paramField, expr)
    
    targetY : oneOf(scaleField, paramField, expr)
    
    """
    _schema = {'$ref': '#/defs/linkpathTransform'}
    _rootschema = Root._schema

    def __init__(self, type=Undefined, orient=Undefined, shape=Undefined, signal=Undefined,
                 sourceX=Undefined, sourceY=Undefined, targetX=Undefined, targetY=Undefined, **kwds):
        super(linkpathTransform, self).__init__(type=type, orient=orient, shape=shape, signal=signal,
                                                sourceX=sourceX, sourceY=sourceY, targetX=targetX,
                                                targetY=targetY, **kwds)


class pieTransform(VegaSchema):
    """pieTransform schema wrapper
    
    Mapping(required=[type])
    
    Attributes
    ----------
    type : enum('pie')
    
    endAngle : anyOf(float, signal)
    
    field : oneOf(scaleField, paramField, expr)
    
    signal : string
    
    sort : anyOf(boolean, signal)
    
    startAngle : anyOf(float, signal)
    
    """
    _schema = {'$ref': '#/defs/pieTransform'}
    _rootschema = Root._schema

    def __init__(self, type=Undefined, endAngle=Undefined, field=Undefined, signal=Undefined,
                 sort=Undefined, startAngle=Undefined, **kwds):
        super(pieTransform, self).__init__(type=type, endAngle=endAngle, field=field, signal=signal,
                                           sort=sort, startAngle=startAngle, **kwds)


class stackTransform(VegaSchema):
    """stackTransform schema wrapper
    
    Mapping(required=[type])
    
    Attributes
    ----------
    type : enum('stack')
    
    field : oneOf(scaleField, paramField, expr)
    
    groupby : oneOf(List(oneOf(scaleField, paramField, expr)), signal)
    
    offset : anyOf(enum('zero', 'center', 'normalize'), signal)
    
    signal : string
    
    sort : compare
    
    """
    _schema = {'$ref': '#/defs/stackTransform'}
    _rootschema = Root._schema

    def __init__(self, type=Undefined, field=Undefined, groupby=Undefined, offset=Undefined,
                 signal=Undefined, sort=Undefined, **kwds):
        super(stackTransform, self).__init__(type=type, field=field, groupby=groupby, offset=offset,
                                             signal=signal, sort=sort, **kwds)


class contourTransform(VegaSchema):
    """contourTransform schema wrapper
    
    Mapping(required=[type, size])
    
    Attributes
    ----------
    size : oneOf(List(anyOf(float, signal)), signal)
    
    type : enum('contour')
    
    bandwidth : anyOf(float, signal)
    
    cellSize : anyOf(float, signal)
    
    count : anyOf(float, signal)
    
    nice : anyOf(float, signal)
    
    signal : string
    
    thresholds : oneOf(List(anyOf(float, signal)), signal)
    
    values : oneOf(List(anyOf(float, signal)), signal)
    
    x : oneOf(scaleField, paramField, expr)
    
    y : oneOf(scaleField, paramField, expr)
    
    """
    _schema = {'$ref': '#/defs/contourTransform'}
    _rootschema = Root._schema

    def __init__(self, size=Undefined, type=Undefined, bandwidth=Undefined, cellSize=Undefined,
                 count=Undefined, nice=Undefined, signal=Undefined, thresholds=Undefined,
                 values=Undefined, x=Undefined, y=Undefined, **kwds):
        super(contourTransform, self).__init__(size=size, type=type, bandwidth=bandwidth,
                                               cellSize=cellSize, count=count, nice=nice, signal=signal,
                                               thresholds=thresholds, values=values, x=x, y=y, **kwds)


class geojsonTransform(VegaSchema):
    """geojsonTransform schema wrapper
    
    Mapping(required=[type])
    
    Attributes
    ----------
    type : enum('geojson')
    
    fields : oneOf(List(oneOf(scaleField, paramField, expr)), signal)
    
    geojson : oneOf(scaleField, paramField, expr)
    
    signal : string
    
    """
    _schema = {'$ref': '#/defs/geojsonTransform'}
    _rootschema = Root._schema

    def __init__(self, type=Undefined, fields=Undefined, geojson=Undefined, signal=Undefined, **kwds):
        super(geojsonTransform, self).__init__(type=type, fields=fields, geojson=geojson, signal=signal,
                                               **kwds)


class geopathTransform(VegaSchema):
    """geopathTransform schema wrapper
    
    Mapping(required=[type])
    
    Attributes
    ----------
    type : enum('geopath')
    
    field : oneOf(scaleField, paramField, expr)
    
    pointRadius : anyOf(float, signal, expr, paramField)
    
    projection : string
    
    signal : string
    
    """
    _schema = {'$ref': '#/defs/geopathTransform'}
    _rootschema = Root._schema

    def __init__(self, type=Undefined, field=Undefined, pointRadius=Undefined, projection=Undefined,
                 signal=Undefined, **kwds):
        super(geopathTransform, self).__init__(type=type, field=field, pointRadius=pointRadius,
                                               projection=projection, signal=signal, **kwds)


class geopointTransform(VegaSchema):
    """geopointTransform schema wrapper
    
    Mapping(required=[type, projection, fields])
    
    Attributes
    ----------
    fields : oneOf(List(oneOf(scaleField, paramField, expr)), signal)
    
    projection : string
    
    type : enum('geopoint')
    
    signal : string
    
    """
    _schema = {'$ref': '#/defs/geopointTransform'}
    _rootschema = Root._schema

    def __init__(self, fields=Undefined, projection=Undefined, type=Undefined, signal=Undefined, **kwds):
        super(geopointTransform, self).__init__(fields=fields, projection=projection, type=type,
                                                signal=signal, **kwds)


class geoshapeTransform(VegaSchema):
    """geoshapeTransform schema wrapper
    
    Mapping(required=[type])
    
    Attributes
    ----------
    type : enum('geoshape')
    
    field : oneOf(scaleField, paramField, expr)
    
    pointRadius : anyOf(float, signal, expr, paramField)
    
    projection : string
    
    signal : string
    
    """
    _schema = {'$ref': '#/defs/geoshapeTransform'}
    _rootschema = Root._schema

    def __init__(self, type=Undefined, field=Undefined, pointRadius=Undefined, projection=Undefined,
                 signal=Undefined, **kwds):
        super(geoshapeTransform, self).__init__(type=type, field=field, pointRadius=pointRadius,
                                                projection=projection, signal=signal, **kwds)


class graticuleTransform(VegaSchema):
    """graticuleTransform schema wrapper
    
    Mapping(required=[type])
    
    Attributes
    ----------
    type : enum('graticule')
    
    extent : oneOf(List(Mapping(required=[])), signal)
    
    extentMajor : oneOf(List(Mapping(required=[])), signal)
    
    extentMinor : oneOf(List(Mapping(required=[])), signal)
    
    precision : anyOf(float, signal)
    
    signal : string
    
    step : oneOf(List(anyOf(float, signal)), signal)
    
    stepMajor : oneOf(List(anyOf(float, signal)), signal)
    
    stepMinor : oneOf(List(anyOf(float, signal)), signal)
    
    """
    _schema = {'$ref': '#/defs/graticuleTransform'}
    _rootschema = Root._schema

    def __init__(self, type=Undefined, extent=Undefined, extentMajor=Undefined, extentMinor=Undefined,
                 precision=Undefined, signal=Undefined, step=Undefined, stepMajor=Undefined,
                 stepMinor=Undefined, **kwds):
        super(graticuleTransform, self).__init__(type=type, extent=extent, extentMajor=extentMajor,
                                                 extentMinor=extentMinor, precision=precision,
                                                 signal=signal, step=step, stepMajor=stepMajor,
                                                 stepMinor=stepMinor, **kwds)


class forceTransform(VegaSchema):
    """forceTransform schema wrapper
    
    Mapping(required=[type])
    
    Attributes
    ----------
    type : enum('force')
    
    alpha : anyOf(float, signal)
    
    alphaMin : anyOf(float, signal)
    
    alphaTarget : anyOf(float, signal)
    
    forces : List(oneOf(Mapping(required=[force]), Mapping(required=[force]), 
    Mapping(required=[force]), Mapping(required=[force]), Mapping(required=[force]), 
    Mapping(required=[force])))
    
    iterations : anyOf(float, signal)
    
    restart : anyOf(boolean, signal)
    
    signal : string
    
    static : anyOf(boolean, signal)
    
    velocityDecay : anyOf(float, signal)
    
    """
    _schema = {'$ref': '#/defs/forceTransform'}
    _rootschema = Root._schema

    def __init__(self, type=Undefined, alpha=Undefined, alphaMin=Undefined, alphaTarget=Undefined,
                 forces=Undefined, iterations=Undefined, restart=Undefined, signal=Undefined,
                 static=Undefined, velocityDecay=Undefined, **kwds):
        super(forceTransform, self).__init__(type=type, alpha=alpha, alphaMin=alphaMin,
                                             alphaTarget=alphaTarget, forces=forces,
                                             iterations=iterations, restart=restart, signal=signal,
                                             static=static, velocityDecay=velocityDecay, **kwds)


class nestTransform(VegaSchema):
    """nestTransform schema wrapper
    
    Mapping(required=[type])
    
    Attributes
    ----------
    type : enum('nest')
    
    generate : anyOf(boolean, signal)
    
    key : oneOf(scaleField, paramField, expr)
    
    keys : oneOf(List(oneOf(scaleField, paramField, expr)), signal)
    
    signal : string
    
    """
    _schema = {'$ref': '#/defs/nestTransform'}
    _rootschema = Root._schema

    def __init__(self, type=Undefined, generate=Undefined, key=Undefined, keys=Undefined,
                 signal=Undefined, **kwds):
        super(nestTransform, self).__init__(type=type, generate=generate, key=key, keys=keys,
                                            signal=signal, **kwds)


class packTransform(VegaSchema):
    """packTransform schema wrapper
    
    Mapping(required=[type])
    
    Attributes
    ----------
    type : enum('pack')
    
    field : oneOf(scaleField, paramField, expr)
    
    padding : anyOf(float, signal)
    
    radius : oneOf(scaleField, paramField, expr)
    
    signal : string
    
    size : oneOf(List(anyOf(float, signal)), signal)
    
    sort : compare
    
    """
    _schema = {'$ref': '#/defs/packTransform'}
    _rootschema = Root._schema

    def __init__(self, type=Undefined, field=Undefined, padding=Undefined, radius=Undefined,
                 signal=Undefined, size=Undefined, sort=Undefined, **kwds):
        super(packTransform, self).__init__(type=type, field=field, padding=padding, radius=radius,
                                            signal=signal, size=size, sort=sort, **kwds)


class partitionTransform(VegaSchema):
    """partitionTransform schema wrapper
    
    Mapping(required=[type])
    
    Attributes
    ----------
    type : enum('partition')
    
    field : oneOf(scaleField, paramField, expr)
    
    padding : anyOf(float, signal)
    
    round : anyOf(boolean, signal)
    
    signal : string
    
    size : oneOf(List(anyOf(float, signal)), signal)
    
    sort : compare
    
    """
    _schema = {'$ref': '#/defs/partitionTransform'}
    _rootschema = Root._schema

    def __init__(self, type=Undefined, field=Undefined, padding=Undefined, round=Undefined,
                 signal=Undefined, size=Undefined, sort=Undefined, **kwds):
        super(partitionTransform, self).__init__(type=type, field=field, padding=padding, round=round,
                                                 signal=signal, size=size, sort=sort, **kwds)


class stratifyTransform(VegaSchema):
    """stratifyTransform schema wrapper
    
    Mapping(required=[type, key, parentKey])
    
    Attributes
    ----------
    key : oneOf(scaleField, paramField, expr)
    
    parentKey : oneOf(scaleField, paramField, expr)
    
    type : enum('stratify')
    
    signal : string
    
    """
    _schema = {'$ref': '#/defs/stratifyTransform'}
    _rootschema = Root._schema

    def __init__(self, key=Undefined, parentKey=Undefined, type=Undefined, signal=Undefined, **kwds):
        super(stratifyTransform, self).__init__(key=key, parentKey=parentKey, type=type, signal=signal,
                                                **kwds)


class treeTransform(VegaSchema):
    """treeTransform schema wrapper
    
    Mapping(required=[type])
    
    Attributes
    ----------
    type : enum('tree')
    
    field : oneOf(scaleField, paramField, expr)
    
    method : anyOf(enum('tidy', 'cluster'), signal)
    
    nodeSize : oneOf(List(anyOf(float, signal)), signal)
    
    signal : string
    
    size : oneOf(List(anyOf(float, signal)), signal)
    
    sort : compare
    
    """
    _schema = {'$ref': '#/defs/treeTransform'}
    _rootschema = Root._schema

    def __init__(self, type=Undefined, field=Undefined, method=Undefined, nodeSize=Undefined,
                 signal=Undefined, size=Undefined, sort=Undefined, **kwds):
        super(treeTransform, self).__init__(type=type, field=field, method=method, nodeSize=nodeSize,
                                            signal=signal, size=size, sort=sort, **kwds)


class treelinksTransform(VegaSchema):
    """treelinksTransform schema wrapper
    
    Mapping(required=[type])
    
    Attributes
    ----------
    type : enum('treelinks')
    
    key : oneOf(scaleField, paramField, expr)
    
    signal : string
    
    """
    _schema = {'$ref': '#/defs/treelinksTransform'}
    _rootschema = Root._schema

    def __init__(self, type=Undefined, key=Undefined, signal=Undefined, **kwds):
        super(treelinksTransform, self).__init__(type=type, key=key, signal=signal, **kwds)


class treemapTransform(VegaSchema):
    """treemapTransform schema wrapper
    
    Mapping(required=[type])
    
    Attributes
    ----------
    type : enum('treemap')
    
    field : oneOf(scaleField, paramField, expr)
    
    method : anyOf(enum('squarify', 'resquarify', 'binary', 'dice', 'slice', 'slicedice'), 
    signal)
    
    padding : anyOf(float, signal)
    
    paddingBottom : anyOf(float, signal)
    
    paddingInner : anyOf(float, signal)
    
    paddingLeft : anyOf(float, signal)
    
    paddingOuter : anyOf(float, signal)
    
    paddingRight : anyOf(float, signal)
    
    paddingTop : anyOf(float, signal)
    
    ratio : anyOf(float, signal)
    
    round : anyOf(boolean, signal)
    
    signal : string
    
    size : oneOf(List(anyOf(float, signal)), signal)
    
    sort : compare
    
    """
    _schema = {'$ref': '#/defs/treemapTransform'}
    _rootschema = Root._schema

    def __init__(self, type=Undefined, field=Undefined, method=Undefined, padding=Undefined,
                 paddingBottom=Undefined, paddingInner=Undefined, paddingLeft=Undefined,
                 paddingOuter=Undefined, paddingRight=Undefined, paddingTop=Undefined, ratio=Undefined,
                 round=Undefined, signal=Undefined, size=Undefined, sort=Undefined, **kwds):
        super(treemapTransform, self).__init__(type=type, field=field, method=method, padding=padding,
                                               paddingBottom=paddingBottom, paddingInner=paddingInner,
                                               paddingLeft=paddingLeft, paddingOuter=paddingOuter,
                                               paddingRight=paddingRight, paddingTop=paddingTop,
                                               ratio=ratio, round=round, signal=signal, size=size,
                                               sort=sort, **kwds)


class voronoiTransform(VegaSchema):
    """voronoiTransform schema wrapper
    
    Mapping(required=[type, x, y])
    
    Attributes
    ----------
    type : enum('voronoi')
    
    x : oneOf(scaleField, paramField, expr)
    
    y : oneOf(scaleField, paramField, expr)
    
    extent : oneOf(List(Mapping(required=[])), signal)
    
    signal : string
    
    size : oneOf(List(anyOf(float, signal)), signal)
    
    """
    _schema = {'$ref': '#/defs/voronoiTransform'}
    _rootschema = Root._schema

    def __init__(self, type=Undefined, x=Undefined, y=Undefined, extent=Undefined, signal=Undefined,
                 size=Undefined, **kwds):
        super(voronoiTransform, self).__init__(type=type, x=x, y=y, extent=extent, signal=signal,
                                               size=size, **kwds)


class wordcloudTransform(VegaSchema):
    """wordcloudTransform schema wrapper
    
    Mapping(required=[type])
    
    Attributes
    ----------
    type : enum('wordcloud')
    
    font : anyOf(string, signal, expr, paramField)
    
    fontSize : anyOf(float, signal, expr, paramField)
    
    fontSizeRange : oneOf(List(anyOf(float, signal)), signal, None)
    
    fontStyle : anyOf(string, signal, expr, paramField)
    
    fontWeight : anyOf(string, signal, expr, paramField)
    
    padding : anyOf(float, signal, expr, paramField)
    
    rotate : anyOf(float, signal, expr, paramField)
    
    signal : string
    
    size : oneOf(List(anyOf(float, signal)), signal)
    
    spiral : anyOf(string, signal)
    
    text : oneOf(scaleField, paramField, expr)
    
    """
    _schema = {'$ref': '#/defs/wordcloudTransform'}
    _rootschema = Root._schema

    def __init__(self, type=Undefined, font=Undefined, fontSize=Undefined, fontSizeRange=Undefined,
                 fontStyle=Undefined, fontWeight=Undefined, padding=Undefined, rotate=Undefined,
                 signal=Undefined, size=Undefined, spiral=Undefined, text=Undefined, **kwds):
        super(wordcloudTransform, self).__init__(type=type, font=font, fontSize=fontSize,
                                                 fontSizeRange=fontSizeRange, fontStyle=fontStyle,
                                                 fontWeight=fontWeight, padding=padding, rotate=rotate,
                                                 signal=signal, size=size, spiral=spiral, text=text,
                                                 **kwds)


class crossfilterTransform(VegaSchema):
    """crossfilterTransform schema wrapper
    
    Mapping(required=[type, fields, query])
    
    Attributes
    ----------
    fields : oneOf(List(oneOf(scaleField, paramField, expr)), signal)
    
    query : oneOf(List(Mapping(required=[])), signal)
    
    type : enum('crossfilter')
    
    signal : string
    
    """
    _schema = {'$ref': '#/defs/crossfilterTransform'}
    _rootschema = Root._schema

    def __init__(self, fields=Undefined, query=Undefined, type=Undefined, signal=Undefined, **kwds):
        super(crossfilterTransform, self).__init__(fields=fields, query=query, type=type, signal=signal,
                                                   **kwds)


class resolvefilterTransform(VegaSchema):
    """resolvefilterTransform schema wrapper
    
    Mapping(required=[type, ignore, filter])
    
    Attributes
    ----------
    filter : Mapping(required=[])
    
    ignore : anyOf(float, signal)
    
    type : enum('resolvefilter')
    
    signal : string
    
    """
    _schema = {'$ref': '#/defs/resolvefilterTransform'}
    _rootschema = Root._schema

    def __init__(self, filter=Undefined, ignore=Undefined, type=Undefined, signal=Undefined, **kwds):
        super(resolvefilterTransform, self).__init__(filter=filter, ignore=ignore, type=type,
                                                     signal=signal, **kwds)


class tickCount(VegaSchema):
    """tickCount schema wrapper
    
    oneOf(float, enum('millisecond', 'second', 'minute', 'hour', 'day', 'week', 'month', 
    'year'), Mapping(required=[interval]), signal)
    """
    _schema = {'$ref': '#/refs/tickCount'}
    _rootschema = Root._schema

    def __init__(self, *args, **kwds):
        super(tickCount, self).__init__(*args, **kwds)


class element(VegaSchema):
    """element schema wrapper
    
    string
    """
    _schema = {'$ref': '#/refs/element'}
    _rootschema = Root._schema

    def __init__(self, *args):
        super(element, self).__init__(*args)


class paramField(VegaSchema):
    """paramField schema wrapper
    
    Mapping(required=[field])
    
    Attributes
    ----------
    field : string
    
    """
    _schema = {'$ref': '#/refs/paramField'}
    _rootschema = Root._schema

    def __init__(self, field=Undefined, **kwds):
        super(paramField, self).__init__(field=field, **kwds)


class field(VegaSchema):
    """field schema wrapper
    
    oneOf(string, oneOf(signal, Mapping(required=[datum]), Mapping(required=[group]), 
    Mapping(required=[parent])))
    """
    _schema = {'$ref': '#/refs/field'}
    _rootschema = Root._schema

    def __init__(self, *args, **kwds):
        super(field, self).__init__(*args, **kwds)


class scale(VegaSchema):
    """scale schema wrapper
    
    oneOf(string, oneOf(signal, Mapping(required=[datum]), Mapping(required=[group]), 
    Mapping(required=[parent])))
    """
    _schema = {'$ref': '#/refs/scale'}
    _rootschema = Root._schema

    def __init__(self, *args, **kwds):
        super(scale, self).__init__(*args, **kwds)


class stringModifiers(VegaSchema):
    """stringModifiers schema wrapper
    
    Mapping(required=[])
    
    Attributes
    ----------
    scale : scale
    
    """
    _schema = {'$ref': '#/refs/stringModifiers'}
    _rootschema = Root._schema

    def __init__(self, scale=Undefined, **kwds):
        super(stringModifiers, self).__init__(scale=scale, **kwds)


class numberModifiers(VegaSchema):
    """numberModifiers schema wrapper
    
    Mapping(required=[])
    
    Attributes
    ----------
    band : anyOf(float, boolean)
    
    exponent : oneOf(float, numberValue)
    
    extra : boolean
    
    mult : oneOf(float, numberValue)
    
    offset : oneOf(float, numberValue)
    
    round : boolean
    
    scale : scale
    
    """
    _schema = {'$ref': '#/refs/numberModifiers'}
    _rootschema = Root._schema

    def __init__(self, band=Undefined, exponent=Undefined, extra=Undefined, mult=Undefined,
                 offset=Undefined, round=Undefined, scale=Undefined, **kwds):
        super(numberModifiers, self).__init__(band=band, exponent=exponent, extra=extra, mult=mult,
                                              offset=offset, round=round, scale=scale, **kwds)


class value(VegaSchema):
    """value schema wrapper
    
    oneOf(List(allOf(rule, allOf(stringModifiers, anyOf(oneOf(signal, Mapping(required=[value]),
     Mapping(required=[field]), Mapping(required=[range])), Mapping(required=[scale, value]), 
    Mapping(required=[scale, band]), Mapping(required=[offset]))))), allOf(stringModifiers, 
    anyOf(oneOf(signal, Mapping(required=[value]), Mapping(required=[field]), 
    Mapping(required=[range])), Mapping(required=[scale, value]), Mapping(required=[scale, 
    band]), Mapping(required=[offset]))))
    """
    _schema = {'$ref': '#/refs/value'}
    _rootschema = Root._schema

    def __init__(self, *args, **kwds):
        super(value, self).__init__(*args, **kwds)


class numberValue(VegaSchema):
    """numberValue schema wrapper
    
    oneOf(List(allOf(rule, allOf(numberModifiers, anyOf(oneOf(signal, Mapping(required=[value]),
     Mapping(required=[field]), Mapping(required=[range])), Mapping(required=[scale, value]), 
    Mapping(required=[scale, band]), Mapping(required=[offset]))))), allOf(numberModifiers, 
    anyOf(oneOf(signal, Mapping(required=[value]), Mapping(required=[field]), 
    Mapping(required=[range])), Mapping(required=[scale, value]), Mapping(required=[scale, 
    band]), Mapping(required=[offset]))))
    """
    _schema = {'$ref': '#/refs/numberValue'}
    _rootschema = Root._schema

    def __init__(self, *args, **kwds):
        super(numberValue, self).__init__(*args, **kwds)


class stringValue(VegaSchema):
    """stringValue schema wrapper
    
    oneOf(List(allOf(rule, allOf(stringModifiers, anyOf(oneOf(signal, Mapping(required=[value]),
     Mapping(required=[field]), Mapping(required=[range])), Mapping(required=[scale, value]), 
    Mapping(required=[scale, band]), Mapping(required=[offset]))))), allOf(stringModifiers, 
    anyOf(oneOf(signal, Mapping(required=[value]), Mapping(required=[field]), 
    Mapping(required=[range])), Mapping(required=[scale, value]), Mapping(required=[scale, 
    band]), Mapping(required=[offset]))))
    """
    _schema = {'$ref': '#/refs/stringValue'}
    _rootschema = Root._schema

    def __init__(self, *args, **kwds):
        super(stringValue, self).__init__(*args, **kwds)


class booleanValue(VegaSchema):
    """booleanValue schema wrapper
    
    oneOf(List(allOf(rule, allOf(stringModifiers, anyOf(oneOf(signal, Mapping(required=[value]),
     Mapping(required=[field]), Mapping(required=[range])), Mapping(required=[scale, value]), 
    Mapping(required=[scale, band]), Mapping(required=[offset]))))), allOf(stringModifiers, 
    anyOf(oneOf(signal, Mapping(required=[value]), Mapping(required=[field]), 
    Mapping(required=[range])), Mapping(required=[scale, value]), Mapping(required=[scale, 
    band]), Mapping(required=[offset]))))
    """
    _schema = {'$ref': '#/refs/booleanValue'}
    _rootschema = Root._schema

    def __init__(self, *args, **kwds):
        super(booleanValue, self).__init__(*args, **kwds)


class arrayValue(VegaSchema):
    """arrayValue schema wrapper
    
    oneOf(List(allOf(rule, allOf(stringModifiers, anyOf(oneOf(signal, Mapping(required=[value]),
     Mapping(required=[field]), Mapping(required=[range])), Mapping(required=[scale, value]), 
    Mapping(required=[scale, band]), Mapping(required=[offset]))))), allOf(stringModifiers, 
    anyOf(oneOf(signal, Mapping(required=[value]), Mapping(required=[field]), 
    Mapping(required=[range])), Mapping(required=[scale, value]), Mapping(required=[scale, 
    band]), Mapping(required=[offset]))))
    """
    _schema = {'$ref': '#/refs/arrayValue'}
    _rootschema = Root._schema

    def __init__(self, *args, **kwds):
        super(arrayValue, self).__init__(*args, **kwds)


class nullableStringValue(VegaSchema):
    """nullableStringValue schema wrapper
    
    oneOf(List(allOf(rule, allOf(stringModifiers, anyOf(oneOf(signal, Mapping(required=[value]),
     Mapping(required=[field]), Mapping(required=[range])), Mapping(required=[scale, value]), 
    Mapping(required=[scale, band]), Mapping(required=[offset]))))), allOf(stringModifiers, 
    anyOf(oneOf(signal, Mapping(required=[value]), Mapping(required=[field]), 
    Mapping(required=[range])), Mapping(required=[scale, value]), Mapping(required=[scale, 
    band]), Mapping(required=[offset]))))
    """
    _schema = {'$ref': '#/refs/nullableStringValue'}
    _rootschema = Root._schema

    def __init__(self, *args, **kwds):
        super(nullableStringValue, self).__init__(*args, **kwds)


class fontWeightValue(VegaSchema):
    """fontWeightValue schema wrapper
    
    oneOf(List(allOf(rule, allOf(stringModifiers, anyOf(oneOf(signal, Mapping(required=[value]),
     Mapping(required=[field]), Mapping(required=[range])), Mapping(required=[scale, value]), 
    Mapping(required=[scale, band]), Mapping(required=[offset]))))), allOf(stringModifiers, 
    anyOf(oneOf(signal, Mapping(required=[value]), Mapping(required=[field]), 
    Mapping(required=[range])), Mapping(required=[scale, value]), Mapping(required=[scale, 
    band]), Mapping(required=[offset]))))
    """
    _schema = {'$ref': '#/refs/fontWeightValue'}
    _rootschema = Root._schema

    def __init__(self, *args, **kwds):
        super(fontWeightValue, self).__init__(*args, **kwds)


class colorRGB(VegaSchema):
    """colorRGB schema wrapper
    
    Mapping(required=[r, g, b])
    
    Attributes
    ----------
    b : numberValue
    
    g : numberValue
    
    r : numberValue
    
    """
    _schema = {'$ref': '#/refs/colorRGB'}
    _rootschema = Root._schema

    def __init__(self, b=Undefined, g=Undefined, r=Undefined, **kwds):
        super(colorRGB, self).__init__(b=b, g=g, r=r, **kwds)


class colorHSL(VegaSchema):
    """colorHSL schema wrapper
    
    Mapping(required=[h, s, l])
    
    Attributes
    ----------
    h : numberValue
    
    l : numberValue
    
    s : numberValue
    
    """
    _schema = {'$ref': '#/refs/colorHSL'}
    _rootschema = Root._schema

    def __init__(self, h=Undefined, l=Undefined, s=Undefined, **kwds):
        super(colorHSL, self).__init__(h=h, l=l, s=s, **kwds)


class colorLAB(VegaSchema):
    """colorLAB schema wrapper
    
    Mapping(required=[l, a, b])
    
    Attributes
    ----------
    a : numberValue
    
    b : numberValue
    
    l : numberValue
    
    """
    _schema = {'$ref': '#/refs/colorLAB'}
    _rootschema = Root._schema

    def __init__(self, a=Undefined, b=Undefined, l=Undefined, **kwds):
        super(colorLAB, self).__init__(a=a, b=b, l=l, **kwds)


class colorHCL(VegaSchema):
    """colorHCL schema wrapper
    
    Mapping(required=[h, c, l])
    
    Attributes
    ----------
    c : numberValue
    
    h : numberValue
    
    l : numberValue
    
    """
    _schema = {'$ref': '#/refs/colorHCL'}
    _rootschema = Root._schema

    def __init__(self, c=Undefined, h=Undefined, l=Undefined, **kwds):
        super(colorHCL, self).__init__(c=c, h=h, l=l, **kwds)


class colorValue(VegaSchema):
    """colorValue schema wrapper
    
    oneOf(nullableStringValue, Mapping(required=[gradient]), Mapping(required=[color]))
    """
    _schema = {'$ref': '#/refs/colorValue'}
    _rootschema = Root._schema

    def __init__(self, *args, **kwds):
        super(colorValue, self).__init__(*args, **kwds)


class expr(VegaSchema):
    """expr schema wrapper
    
    Mapping(required=[expr])
    
    Attributes
    ----------
    expr : string
    
    """
    _schema = {'$ref': '#/refs/expr'}
    _rootschema = Root._schema

    def __init__(self, expr=Undefined, **kwds):
        super(expr, self).__init__(expr=expr, **kwds)


class exprString(VegaSchema):
    """exprString schema wrapper
    
    string
    """
    _schema = {'$ref': '#/refs/exprString'}
    _rootschema = Root._schema

    def __init__(self, *args):
        super(exprString, self).__init__(*args)


class compare(VegaSchema):
    """compare schema wrapper
    
    oneOf(Mapping(required=[]), Mapping(required=[]))
    """
    _schema = {'$ref': '#/refs/compare'}
    _rootschema = Root._schema

    def __init__(self, *args, **kwds):
        super(compare, self).__init__(*args, **kwds)


class from_(VegaSchema):
    """from_ schema wrapper
    
    Mapping(required=[])
    
    Attributes
    ----------
    data : string
    
    """
    _schema = {'$ref': '#/refs/from'}
    _rootschema = Root._schema

    def __init__(self, data=Undefined, **kwds):
        super(from_, self).__init__(data=data, **kwds)


class facet(VegaSchema):
    """facet schema wrapper
    
    Mapping(required=[facet])
    
    Attributes
    ----------
    facet : oneOf(Mapping(required=[name, data, field]), Mapping(required=[name, data, 
    groupby]))
    
    data : string
    
    """
    _schema = {'$ref': '#/refs/facet'}
    _rootschema = Root._schema

    def __init__(self, facet=Undefined, data=Undefined, **kwds):
        super(facet, self).__init__(facet=facet, data=data, **kwds)


class markclip(VegaSchema):
    """markclip schema wrapper
    
    oneOf(booleanOrSignal, Mapping(required=[path]), Mapping(required=[sphere]))
    """
    _schema = {'$ref': '#/refs/markclip'}
    _rootschema = Root._schema

    def __init__(self, *args, **kwds):
        super(markclip, self).__init__(*args, **kwds)


class style(VegaSchema):
    """style schema wrapper
    
    oneOf(string, List(string))
    """
    _schema = {'$ref': '#/refs/style'}
    _rootschema = Root._schema

    def __init__(self, *args, **kwds):
        super(style, self).__init__(*args, **kwds)


class marktype(VegaSchema):
    """marktype schema wrapper
    
    string
    """
    _schema = {'$ref': '#/refs/marktype'}
    _rootschema = Root._schema

    def __init__(self, *args):
        super(marktype, self).__init__(*args)


class sortOrder(VegaSchema):
    """sortOrder schema wrapper
    
    oneOf(enum('ascending', 'descending'), signal)
    """
    _schema = {'$ref': '#/refs/sortOrder'}
    _rootschema = Root._schema

    def __init__(self, *args, **kwds):
        super(sortOrder, self).__init__(*args, **kwds)


class scaleField(VegaSchema):
    """scaleField schema wrapper
    
    oneOf(string, signal)
    """
    _schema = {'$ref': '#/refs/scaleField'}
    _rootschema = Root._schema

    def __init__(self, *args, **kwds):
        super(scaleField, self).__init__(*args, **kwds)


class scaleInterpolate(VegaSchema):
    """scaleInterpolate schema wrapper
    
    oneOf(string, signal, Mapping(required=[type]))
    """
    _schema = {'$ref': '#/refs/scaleInterpolate'}
    _rootschema = Root._schema

    def __init__(self, *args, **kwds):
        super(scaleInterpolate, self).__init__(*args, **kwds)


class scaleData(VegaSchema):
    """scaleData schema wrapper
    
    oneOf(Mapping(required=[data, field]), Mapping(required=[data, fields]), 
    Mapping(required=[fields]))
    """
    _schema = {'$ref': '#/refs/scaleData'}
    _rootschema = Root._schema

    def __init__(self, *args, **kwds):
        super(scaleData, self).__init__(*args, **kwds)


class selector(VegaSchema):
    """selector schema wrapper
    
    string
    """
    _schema = {'$ref': '#/refs/selector'}
    _rootschema = Root._schema

    def __init__(self, *args):
        super(selector, self).__init__(*args)


class signal(VegaSchema):
    """signal schema wrapper
    
    Mapping(required=[signal])
    
    Attributes
    ----------
    signal : string
    
    """
    _schema = {'$ref': '#/refs/signal'}
    _rootschema = Root._schema

    def __init__(self, signal=Undefined, **kwds):
        super(signal, self).__init__(signal=signal, **kwds)


class booleanOrSignal(VegaSchema):
    """booleanOrSignal schema wrapper
    
    oneOf(boolean, signal)
    """
    _schema = {'$ref': '#/refs/booleanOrSignal'}
    _rootschema = Root._schema

    def __init__(self, *args, **kwds):
        super(booleanOrSignal, self).__init__(*args, **kwds)


class numberOrSignal(VegaSchema):
    """numberOrSignal schema wrapper
    
    oneOf(float, signal)
    """
    _schema = {'$ref': '#/refs/numberOrSignal'}
    _rootschema = Root._schema

    def __init__(self, *args, **kwds):
        super(numberOrSignal, self).__init__(*args, **kwds)


class stringOrSignal(VegaSchema):
    """stringOrSignal schema wrapper
    
    oneOf(string, signal)
    """
    _schema = {'$ref': '#/refs/stringOrSignal'}
    _rootschema = Root._schema

    def __init__(self, *args, **kwds):
        super(stringOrSignal, self).__init__(*args, **kwds)




"""
Step Chart
-----------------
This example shows Google's stock price over time.
"""
# category: basic charts

import altair as alt
from altair.expr import datum
from vega_datasets import data

stocks = data.stocks()

chart = alt.Chart(stocks).mark_line(interpolate='step-after').encode(
    x = 'date',
    y = 'price'
).transform_filter(
    datum.symbol =='GOOG'
)



"""
Line chart with Confidence Interval Band
========================================
This example shows how to make a line chart with a confidence interval Band
"""

import altair as alt
from vega_datasets import data

cars = data.cars()

line = alt.Chart(cars).mark_line().encode(
    x='Year', 
    y='mean(Miles_per_Gallon)'
)

confidence_interval = alt.Chart(cars).mark_area(opacity=0.3).encode(
    x='Year', 
    y=alt.Y('ci0(Miles_per_Gallon)', axis=alt.Axis(title='Miles/Gallon')),
    y2='ci1(Miles_per_Gallon)'
)

chart = confidence_interval + line


import json
import warnings

import jsonschema
import six
import pandas as pd

from .schema import core, channels, mixins, Undefined

from .data import data_transformers, pipe
from ... import utils
from .display import renderers


SCHEMA_URL = "https://vega.github.io/schema/vega-lite/v2.json"


#------------------------------------------------------------------------
# Aliases
Bin = core.BinParams

#------------------------------------------------------------------------
# Encoding will contain channel objects that aren't valid at instantiation
core.EncodingWithFacet._class_is_valid_at_instantiation = False

#------------------------------------------------------------------------
# These are parameters that are valid at the top level, but are not valid
# for specs that are within a composite chart
# (layer, hconcat, vconcat, facet, repeat)
TOPLEVEL_ONLY_KEYS = {'background', 'config', 'autosize', 'padding', '$schema'}


def _get_channels_mapping():
    mapping = {}
    for attr in dir(channels):
        cls = getattr(channels, attr)
        if isinstance(cls, type) and issubclass(cls, core.SchemaBase):
            mapping[cls] = attr.replace('Value', '').lower()
    return mapping


# -------------------------------------------------------------------------
# Tools for working with selections
class SelectionMapping(core.VegaLiteSchema):
    """A mapping of selection names to selection definitions"""
    _schema = {
        'type': 'object',
        'additionalPropeties': {'$ref': '#/definitions/SelectionDef'}
    }
    _rootschema = core.Root._schema

    def ref(self, name=None):
        """Return a named selection reference.

        If the mapping contains only one selection, then the name need not
        be specified.
        """
        if name is None and len(self._kwds) == 1:
            name = list(self._kwds.keys())[0]
        if name not in self._kwds:
            raise ValueError("'{0}' is not a valid selection name "
                             "in this mapping".format(name))
        return {"selection": name}

    def _get_name(self):
        if len(self._kwds) != 1:
            raise ValueError("Selection Mapping has more than one name")
        return next(iter(self._kwds))

    def __add__(self, other):
        if isinstance(other, SelectionMapping):
            copy = self.copy()
            copy._kwds.update(other._kwds)
            return copy
        else:
            return NotImplemented

    def __iadd__(self, other):
        if isinstance(other, SelectionMapping):
            self._kwds.update(other._kwds)
            return self
        else:
            return NotImplemented

    def __invert__(self):
        return core.SelectionNot(**{'not': self._get_name()})

    def __and__(self, other):
        if isinstance(other, SelectionMapping):
            other = other._get_name()
        return core.SelectionAnd(**{'and': [self._get_name(), other]})

    def __or__(self, other):
        if isinstance(other, SelectionMapping):
            other = other._get_name()
        return core.SelectionOr(**{'or': [self._get_name(), other]})

#------------------------------------------------------------------------
# Top-Level Functions

def value(value, **kwargs):
    """Specify a value for use in an encoding"""
    return dict(value=value, **kwargs)


def selection(name=None, **kwds):
    """Create a named selection.

    Parameters
    ----------
    name : string (optional)
        The name of the selection. If not specified, a unique name will be
        created.
    **kwds :
        additional keywords will be used to construct a SelectionDef instance
        that controls the selection.

    Returns
    -------
    selection: SelectionMapping
        The SelectionMapping object that can be used in chart creation.
    """
    if name is None:
        name = "selector{0:03d}".format(selection.counter)
        selection.counter += 1
    return SelectionMapping(**{name: core.SelectionDef(**kwds)})

selection.counter = 1


@utils.use_signature(core.IntervalSelection)
def selection_interval(**kwargs):
    """A selection with type='interval'"""
    return selection(type='interval', **kwargs)


@utils.use_signature(core.MultiSelection)
def selection_multi(**kwargs):
    """A selection with type='multi'"""
    return selection(type='multi', **kwargs)


@utils.use_signature(core.SingleSelection)
def selection_single(**kwargs):
    """A selection with type='single'"""
    return selection(type='single', **kwargs)


@utils.use_signature(core.VgBinding)
def binding(input, **kwargs):
    """A generic binding"""
    return core.VgBinding(input=input, **kwargs)


@utils.use_signature(core.VgCheckboxBinding)
def binding_checkbox(**kwargs):
    """A checkbox binding"""
    return core.VgCheckboxBinding(input='checkbox', **kwargs)


@utils.use_signature(core.VgRadioBinding)
def binding_radio(**kwargs):
    """A radio button binding"""
    return core.VgRadioBinding(input='radio', **kwargs)


@utils.use_signature(core.VgSelectBinding)
def binding_select(**kwargs):
    """A select binding"""
    return core.VgSelectBinding(input='select', **kwargs)


@utils.use_signature(core.VgRangeBinding)
def binding_range(**kwargs):
    """A range binding"""
    return core.VgRangeBinding(input='range', **kwargs)


def condition(predicate, if_true, if_false, **kwargs):
    """A conditional attribute or encoding

    Parameters
    ----------
    predicate: SelectionMapping, LogicalOperandPredicate, dict, or string
        the selection predicate or test predicate for the condition.
        if a string is passed, it will be treated as a test operand.
    if_true:
        the spec or object to use if the selection predicate is true
    if_false:
        the spec or object to use if the selection predicate is false
    **kwargs:
        additional keyword args are added to the resulting dict

    Returns
    -------
    spec: dict or VegaLiteSchema
        the spec that describes the condition
    """
    selection_predicates = (core.SelectionNot, core.SelectionOr,
                            core.SelectionAnd, core.SelectionOperand)
    test_predicates = (six.string_types, core.Predicate,
                       core.LogicalOperandPredicate, core.LogicalNotPredicate,
                       core.LogicalOrPredicate, core.LogicalAndPredicate)

    if isinstance(predicate, SelectionMapping):
        condition = {'selection': predicate._get_name()}
    elif isinstance(predicate, selection_predicates):
        condition = {'selection': predicate}
    elif isinstance(predicate, test_predicates):
        condition = {'test': predicate}
    elif isinstance(predicate, dict):
        condition = predicate
    else:
        raise NotImplementedError("condition predicate of type {0}"
                                  "".format(type(predicate)))

    if isinstance(if_true, core.SchemaBase):
        # convert to dict for now; the from_dict call below will wrap this
        # dict in the appropriate schema
        if_true = if_true.to_dict()
    elif isinstance(if_true, six.string_types):
        if_true = {'field': if_true}
        if_true.update(kwargs)
    condition.update(if_true)

    if isinstance(if_false, core.SchemaBase):
        # For the selection, the channel definitions all allow selections
        # already. So use this SchemaBase wrapper if possible.
        selection = if_false.copy()
        selection.condition = condition
    elif isinstance(if_false, six.string_types):
        selection = dict(condition=condition, field=if_false)
        selection.update(kwargs)
    else:
        selection = dict(condition=condition, **if_false)

    return selection


#--------------------------------------------------------------------
# Top-level objects

class TopLevelMixin(mixins.ConfigMethodMixin):
    """Mixin for top-level chart objects such as Chart, LayeredChart, etc."""
    _default_spec_values = {"config": {"view": {"width": 400, "height": 300}}}
    _class_is_valid_at_instantiation = False

    def _prepare_data(self):
        if isinstance(self.data, (dict, core.Data, core.InlineData,
                                  core.UrlData, core.NamedData)):
            pass
        elif isinstance(self.data, pd.DataFrame):
            self.data = pipe(self.data, data_transformers.get())
        elif isinstance(self.data, six.string_types):
            self.data = core.UrlData(self.data)

    def to_dict(self, *args, **kwargs):
        """Convert the chart to a dictionary suitable for JSON export"""
        copy = self.copy()
        original_data = getattr(copy, 'data', Undefined)
        copy._prepare_data()

        # We make use of two context markers:
        # - 'data' points to the data that should be referenced for column type
        #   inference.
        # - 'top_level' is a boolean flag that is assumed to be true; if it's
        #   true then a "$schema" arg is added to the dict.
        context = kwargs.get('context', {}).copy()

        is_top_level = context.get('top_level', True)
        context['top_level'] = False

        if original_data is not Undefined:
            context['data'] = original_data
        kwargs['context'] = context

        try:
            dct = super(TopLevelMixin, copy).to_dict(*args, **kwargs)
        except jsonschema.ValidationError:
            dct = None

        # If we hit an error, then re-convert with validate='deep' to get
        # a more useful traceback. We don't do this by default because it's
        # much slower in the case that there are no errors.
        if dct is None:
            kwargs['validate'] = 'deep'
            dct = super(TopLevelMixin, copy).to_dict(*args, **kwargs)

        if is_top_level:
            # since this is top-level we add $schema if it's missing
            if '$schema' not in dct:
                dct['$schema'] = SCHEMA_URL

            # add default values if present
            if copy._default_spec_values:
                dct = utils.update_nested(copy._default_spec_values, dct, copy=True)
        return dct

    def savechart(self, fp, format=None, **kwargs):
        """Save a chart to file in a variety of formats

        Supported formats are json, html, png, svg

        Parameters
        ----------
        fp : string filename or file-like object
            file in which to write the chart.
        format : string (optional)
            the format to write: one of ['json', 'html', 'png', 'eps'].
            If not specified, the format will be determined from the filename.
        **kwargs :
            Additional keyword arguments are passed to the output method
            associated with the specified format.
        """
        if isinstance(fp, six.string_types):
            format = fp.split('.')[-1]

        if format is None:
            raise ValueError("must specify file format: "
                             "['png', 'eps', 'html', 'json']")
        elif format == 'json':
            utils.write_file_or_filename(fp, self.to_json(**kwargs), mode='w')
        elif format == 'html':
            from .html import HTML_TEMPLATE
            opt = dict(renderer=kwargs.pop('renderer', 'canvas'),
                       actions=kwargs.pop('actions', False))
            if opt['renderer'] not in ('canvas', 'svg'):
                raise ValueError("renderer must be 'canvas' or 'svg'")
            spec_html = HTML_TEMPLATE.format(spec=self.to_json(**kwargs),
                                             opt=json.dumps(opt))
            utils.write_file_or_filename(fp, spec_html, mode='w')
        elif format in ['png', 'svg']:
            utils.save_spec(self.to_dict(), fp, format=format, **kwargs)
        else:
            raise ValueError("unrecognized format: '{0}'".format(format))

    # Layering and stacking

    def __add__(self, other):
        return LayerChart([self, other])

    def __and__(self, other):
        return VConcatChart([self, other])

    def __or__(self, other):
        return HConcatChart([self, other])

    # Display-related methods

    def _repr_mimebundle_(self, include, exclude):
        """Return a MIME bundle for display in Jupyter frontends."""
        # Catch errors explicitly to get around issues in Jupyter frontend
        # see https://github.com/ipython/ipython/issues/11038
        try:
            dct = self.to_dict()
        except Exception:
            utils.display_traceback(in_ipython=True)
            return {}
        else:
            return renderers.get()(dct)

    def repeat(self, row=Undefined, column=Undefined, **kwargs):
        """Return a RepeatChart built from the chart

        Fields within the chart can be set to correspond to the row or
        column using `alt.repeat('row')` and `alt.repeat('column')`.

        Parameters
        ----------
        row : list
            a list of data column names to be mapped to the row facet
        column : list
            a list of data column names to be mapped to the column facet

        Returns
        -------
        chart : RepeatChart
            a repeated chart.
        """
        repeat = core.Repeat(row=row, column=column)
        return RepeatChart(spec=self, repeat=repeat, **kwargs)

    def properties(self, **kwargs):
        """Set top-level properties of the Chart.

        Argument names and types are the same as class initialization.
        """
        copy = self.copy(deep=True, ignore=['data'])
        for key, val in kwargs.items():
            setattr(copy, key, val)
        return copy

    def _add_transform(self, *transforms):
        """Copy the chart and add specified transforms to chart.transform"""
        copy = self.copy()
        if copy.transform is Undefined:
            copy.transform = list(transforms)
        else:
            copy.transform.extend(transforms)
        return copy

    def transform_aggregate(self, aggregate, groupby=Undefined, **kwargs):
        """
        Add an AggregateTransform to the schema.

        Attributes
        ----------
        aggregate : List(AggregatedFieldDef)
            Array of objects that define fields to aggregate.
        groupby : List(string)
            The data fields to group by. If not specified, a single group
            containing all data objects will be used.

        Returns
        -------
        self : Chart object
            returns chart to allow for chaining

        See Also
        --------
        alt.AggregateTransform : underlying transform object
        """
        kwargs['aggregate'] = aggregate
        kwargs['groupby'] = groupby
        return self._add_transform(core.AggregateTransform(**kwargs))

    def transform_bin(self, as_=Undefined, bin=Undefined, field=Undefined, **kwargs):
        """
        Add a BinTransform to the schema.

        Attributes
        ----------
        as_ : string
            The output fields at which to write the start and end bin values.
        bin : anyOf(boolean, BinParams)
            An object indicating bin properties, or simply `true` for using
            default bin parameters.
        field : string
            The data field to bin.

        Returns
        -------
        self : Chart object
            returns chart to allow for chaining

        See Also
        --------
        alt.BinTransform : underlying transform object
        """
        if as_ is not Undefined:
            if 'as' in kwargs:
                raise ValueError("transform_bin: both 'as_' and 'as' passed as arguments.")
            kwargs['as'] = as_
        kwargs['bin'] = bin
        kwargs['field'] = field
        return self._add_transform(core.BinTransform(**kwargs))

    def transform_calculate(self, as_=Undefined, calculate=Undefined, **kwargs):
        """
        Add a CalculateTransform to the schema.

        Attributes
        ----------
        as_ : string
            The output fields at which to write the start and end bin values.
        calculate : string or alt.expr expression
            An expression string. Use the variable `datum` to refer to the
            current data object.

        Returns
        -------
        self : Chart object
            returns chart to allow for chaining

        See Also
        --------
        alt.CalculateTransform : underlying transform object
        """
        if as_ is not Undefined:
            if 'as' in kwargs:
                raise ValueError("transform_calculate: both 'as_' and 'as' passed as arguments.")
            kwargs['as'] = as_
        kwargs['calculate'] = calculate
        return self._add_transform(core.CalculateTransform(**kwargs))

    def transform_filter(self, filter, **kwargs):
        """
        Add a FilterTransform to the schema.

        Attributes
        ----------
        filter : LogicalOperandPredicate
            The `filter` property must be one of the predicate definitions:
            (1) a string or alt.expr expression
            (2) a range predicate
            (3) a selection predicate
            (4) a logical operand combining (1)-(3)

        Returns
        -------
        self : Chart object
            returns chart to allow for chaining

        See Also
        --------
        alt.FilterTransform : underlying transform object
        """
        return self._add_transform(core.FilterTransform(filter=filter, **kwargs))

    def transform_lookup(self, from_=Undefined, lookup=Undefined, default=Undefined, **kwargs):
        """Add a LookupTransform to the schema

        Attributes
        ----------
        from_ : LookupData
            Secondary data reference.
        lookup : string
            Key in primary data source.
        default : string
            The default value to use if lookup fails

        Returns
        -------
        self : Chart object
            returns chart to allow for chaining

        See Also
        --------
        alt.LookupTransform : underlying transform object
        """
        if from_ is not Undefined:
            if 'from' in kwargs:
                raise ValueError("transform_lookup: both 'from_' and 'from' passed as arguments.")
            kwargs['from'] = from_
        kwargs['lookup'] = lookup
        kwargs['default'] = default
        return self._add_transform(core.LookupTransform(**kwargs))

    def transform_timeunit(self, as_=Undefined, field=Undefined, timeUnit=Undefined, **kwargs):
        """
        Add a TimeUnitTransform to the schema.

        Attributes
        ----------
        as_ : string
            The output fields at which to write the start and end bin values.
        field : string
            The data field to apply time unit.
        timeUnit : TimeUnit
            The timeUnit.

        Returns
        -------
        self : Chart object
            returns chart to allow for chaining

        See Also
        --------
        alt.TimeUnitTransform : underlying transform object
        """
        if as_ is not Undefined:
            if 'as' in kwargs:
                raise ValueError("transform_timeunit: both 'as_' and 'as' passed as arguments.")
            kwargs['as'] = as_
        kwargs['field'] = field
        kwargs['timeUnit'] = timeUnit
        return self._add_transform(core.TimeUnitTransform(**kwargs))

    @utils.use_signature(core.Resolve)
    def _set_resolve(self, **kwargs):
        """Copy the chart and update the resolve property with kwargs"""
        copy = self.copy()
        if copy.resolve is Undefined:
            copy.resolve = core.Resolve()
        for key, val in kwargs.items():
            copy.resolve[key] = val
        return copy

    @utils.use_signature(core.AxisResolveMap)
    def resolve_axis(self, *args, **kwargs):
        return self._set_resolve(axis=core.AxisResolveMap(*args, **kwargs))

    @utils.use_signature(core.LegendResolveMap)
    def resolve_legend(self, *args, **kwargs):
        return self._set_resolve(legend=core.LegendResolveMap(*args, **kwargs))

    @utils.use_signature(core.ScaleResolveMap)
    def resolve_scale(self, *args, **kwargs):
        return self._set_resolve(scale=core.ScaleResolveMap(*args, **kwargs))


class Chart(TopLevelMixin, mixins.MarkMethodMixin, core.TopLevelFacetedUnitSpec):
    """Create a basic Altair/Vega-Lite chart.

    Although it is possible to set all Chart properties as constructor attributes,
    it is more idiomatic to use methods such as ``mark_point()``, ``encode()``,
    ``transform_filter()``, ``properties()``, etc. See Altair's documentation
    for details and examples: http://altair-viz.github.io/.

    Attributes
    ----------
    data : Data
        An object describing the data source
    mark : AnyMark
        A string describing the mark type (one of `"bar"`, `"circle"`, `"square"`, `"tick"`,
         `"line"`, * `"area"`, `"point"`, `"rule"`, `"geoshape"`, and `"text"`) or a
         MarkDef object.
    encoding : EncodingWithFacet
        A key-value mapping between encoding channels and definition of fields.
    autosize : anyOf(AutosizeType, AutoSizeParams)
        Sets how the visualization size should be determined. If a string, should be one of
        `"pad"`, `"fit"` or `"none"`. Object values can additionally specify parameters for
        content sizing and automatic resizing. `"fit"` is only supported for single and
        layered views that don't use `rangeStep`.  __Default value__: `pad`
    background : string
        CSS color property to use as the background of visualization.  __Default value:__
        none (transparent)
    config : Config
        Vega-Lite configuration object.  This property can only be defined at the top-level
        of a specification.
    description : string
        Description of this mark for commenting purpose.
    height : float
        The height of a visualization.
    name : string
        Name of the visualization for later reference.
    padding : Padding
        The default visualization padding, in pixels, from the edge of the visualization
        canvas to the data rectangle.  If a number, specifies padding for all sides. If an
        object, the value should have the format `{"left": 5, "top": 5, "right": 5,
        "bottom": 5}` to specify padding for each side of the visualization.  __Default
        value__: `5`
    projection : Projection
        An object defining properties of geographic projection.  Works with `"geoshape"`
        marks and `"point"` or `"line"` marks that have a channel (one or more of `"X"`,
        `"X2"`, `"Y"`, `"Y2"`) with type `"latitude"`, or `"longitude"`.
    selection : Mapping(required=[])
        A key-value mapping between selection names and definitions.
    title : anyOf(string, TitleParams)
        Title for the plot.
    transform : List(Transform)
        An array of data transformations such as filter and new field calculation.
    width : float
        The width of a visualization.
    """
    def __init__(self, data=Undefined, encoding=Undefined, mark=Undefined,
                 width=Undefined, height=Undefined, **kwargs):
        super(Chart, self).__init__(data=data, encoding=encoding, mark=mark,
                                    width=width, height=height, **kwargs)

    @utils.use_signature(core.EncodingWithFacet)
    def encode(self, *args, **kwargs):
        # First convert args to kwargs by inferring the class from the argument
        if args:
            mapping = _get_channels_mapping()
            for arg in args:
                encoding = mapping.get(type(arg), None)
                if encoding is None:
                    raise NotImplementedError("non-keyword arg of type {0}"
                                              "".format(type(arg)))
                if encoding in kwargs:
                    raise ValueError("encode: encoding {0} specified twice"
                                     "".format(encoding))
                kwargs[encoding] = arg

        def _wrap_in_channel_class(obj, prop):
            clsname = prop.title()

            if isinstance(obj, core.SchemaBase):
                return obj

            if isinstance(obj, six.string_types):
                obj = {'field': obj}

            # if obj is not a string or Schema, it must be a mapping
            if 'field' in obj:
                obj = obj.copy()
                obj.update(utils.parse_shorthand(obj['field']))

            if 'value' in obj:
                clsname += 'Value'

            try:
                cls = getattr(channels, clsname)
            except AttributeError:
                raise ValueError("Unrecognized encoding channel '{0}'".format(prop))

            try:
                # Don't force validation here; some objects won't be valid until
                # they're created in the context of a chart.
                return cls.from_dict(obj, validate=False)
            except jsonschema.ValidationError:
                # our attempts at finding the correct class have failed
                return obj

        for prop, field in list(kwargs.items()):
            try:
                condition = field['condition']
            except (KeyError, TypeError):
                pass
            else:
                if condition is not Undefined:
                    field['condition'] = _wrap_in_channel_class(condition, prop)
            kwargs[prop] = _wrap_in_channel_class(field, prop)

        copy = self.copy(deep=True, ignore=['data'])

        # get a copy of the dict representation of the previous encoding
        encoding = copy.encoding
        if encoding is Undefined:
            encoding = {}
        elif isinstance(encoding, dict):
            pass
        else:
            encoding = {k: v for k, v in encoding._kwds.items()
                        if v is not Undefined}

        # update with the new encodings, and apply them to the copy
        encoding.update(kwargs)
        copy.encoding = core.EncodingWithFacet(**encoding)
        return copy

    def interactive(self, name=None, bind_x=True, bind_y=True):
        """Make chart axes scales interactive

        Parameters
        ----------
        name : string
            The selection name to use for the axes scales. This name should be
            unique among all selections within the chart.
        bind_x : boolean, default True
            If true, then bind the interactive scales to the x-axis
        bind_y : boolean, default True
            If true, then bind the interactive scales to the y-axis

        Returns
        -------
        chart :
            copy of self, with interactive axes added
        """
        encodings = []
        if bind_x:
            encodings.append('x')
        if bind_y:
            encodings.append('y')
        copy = self.copy(deep=True, ignore=['data'])

        if copy.selection is Undefined:
            copy.selection = SelectionMapping()
        if isinstance(copy.selection, dict):
            copy.selection = SelectionMapping(**copy.selection)
        copy.selection += selection(type='interval', bind='scales',
                                    encodings=encodings)
        return copy

    def facet(self, row=Undefined, column=Undefined, data=Undefined, **kwargs):
        """Create a facet chart from the current chart.

        Faceted charts require data to be specified at the top level; if data
        is not specified, the data from the current chart will be used at the
        top level.
        """
        if data is Undefined:
            data = self.data
            self = self.copy()
            self.data = Undefined
        return FacetChart(self, row=row, column=column, data=data, **kwargs)



def _check_if_valid_subspec(spec, classname):
    """Check if the spec is a valid sub-spec.

    If it is not, then raise a ValueError
    """
    err = ('Objects with "{0}" attribute cannot be used within {1}. '
           'Consider defining the {0} attribute in the {1} object instead.')

    for attr in TOPLEVEL_ONLY_KEYS:
        if isinstance(spec, core.SchemaBase):
            val = getattr(spec, attr, Undefined)
        else:
            val = spec.get(attr, Undefined)
        if val is not Undefined:
            raise ValueError(err.format(attr, classname))


@utils.use_signature(core.TopLevelRepeatSpec)
class RepeatChart(TopLevelMixin, core.TopLevelRepeatSpec):
    """A chart repeated across rows and columns with small changes"""
    def __init__(self, spec=Undefined, data=Undefined, repeat=Undefined, **kwargs):
        _check_if_valid_subspec(spec, 'RepeatChart')
        super(RepeatChart, self).__init__(spec=spec, data=data, repeat=repeat, **kwargs)

    def interactive(self, name=None, bind_x=True, bind_y=True):
        """Make chart axes scales interactive

        Parameters
        ----------
        name : string
            The selection name to use for the axes scales. This name should be
            unique among all selections within the chart.
        bind_x : boolean, default True
            If true, then bind the interactive scales to the x-axis
        bind_y : boolean, default True
            If true, then bind the interactive scales to the y-axis

        Returns
        -------
        chart :
            copy of self, with interactive axes added
        """
        copy = self.copy()
        copy.spec = copy.spec.interactive(name=name, bind_x=bind_x, bind_y=bind_y)
        return copy


def repeat(repeater):
    """Tie a channel to the row or column within a repeated chart

    The output of this should be passed to the ``field`` attribute of
    a channel.

    Parameters
    ----------
    repeater : {'row'|'column'}
        The repeater to tie the field to.

    Returns
    -------
    repeat : RepeatRef object
    """
    assert repeater in ['row', 'column']
    return core.RepeatRef(repeat=repeater)


@utils.use_signature(core.TopLevelHConcatSpec)
class HConcatChart(TopLevelMixin, core.TopLevelHConcatSpec):
    """A chart with horizontally-concatenated facets"""
    def __init__(self, hconcat=(), **kwargs):
        # TODO: move common data to top level?
        for spec in hconcat:
            _check_if_valid_subspec(spec, 'HConcatChart')
        super(HConcatChart, self).__init__(hconcat=list(hconcat), **kwargs)

    def __ior__(self, other):
        _check_if_valid_subspec(other, 'HConcatChart')
        self.hconcat.append(other)
        return self

    def __or__(self, other):
        _check_if_valid_subspec(other, 'HConcatChart')
        copy = self.copy()
        copy.hconcat.append(other)
        return copy

    # TODO: think about the most useful class API here


def hconcat(*charts, **kwargs):
    """Concatenate charts horizontally"""
    return HConcatChart(hconcat=charts, **kwargs)


@utils.use_signature(core.TopLevelVConcatSpec)
class VConcatChart(TopLevelMixin, core.TopLevelVConcatSpec):
    """A chart with vertically-concatenated facets"""
    def __init__(self, vconcat=(), **kwargs):
        # TODO: move common data to top level?
        for spec in vconcat:
            _check_if_valid_subspec(spec, 'VConcatChart')
        super(VConcatChart, self).__init__(vconcat=list(vconcat), **kwargs)

    def __iand__(self, other):
        _check_if_valid_subspec(other, 'VConcatChart')
        self.vconcat.append(other)
        return self

    def __and__(self, other):
        _check_if_valid_subspec(other, 'VConcatChart')
        copy = self.copy()
        copy.vconcat.append(other)
        return copy

    # TODO: think about the most useful class API here


def vconcat(*charts, **kwargs):
    """Concatenate charts vertically"""
    return VConcatChart(vconcat=charts, **kwargs)


@utils.use_signature(core.TopLevelLayerSpec)
class LayerChart(TopLevelMixin, core.TopLevelLayerSpec):
    """A Chart with layers within a single panel"""
    def __init__(self, layer=(), **kwargs):
        # TODO: move common data to top level?
        # TODO: check for conflicting interaction
        for spec in layer:
            _check_if_valid_subspec(spec, 'LayerChart')
        super(LayerChart, self).__init__(layer=list(layer), **kwargs)

    def __iadd__(self, other):
        _check_if_valid_subspec(other, 'LayerChart')
        self.layer.append(other)
        return self

    def __add__(self, other):
        _check_if_valid_subspec(other, 'LayerChart')
        copy = self.copy()
        copy.layer.append(other)
        return copy

    def interactive(self, name=None, bind_x=True, bind_y=True):
        """Make chart axes scales interactive

        Parameters
        ----------
        name : string
            The selection name to use for the axes scales. This name should be
            unique among all selections within the chart.
        bind_x : boolean, default True
            If true, then bind the interactive scales to the x-axis
        bind_y : boolean, default True
            If true, then bind the interactive scales to the y-axis

        Returns
        -------
        chart :
            copy of self, with interactive axes added
        """
        if not self.layer:
            raise ValueError("LayerChart: cannot call interactive() until a "
                             "layer is defined")
        copy = self.copy()
        copy.layer[0] = copy.layer[0].interactive(name=name, bind_x=bind_x, bind_y=bind_y)
        return copy

    def facet(self, row=Undefined, column=Undefined, data=Undefined, **kwargs):
        """Create a facet chart from the current chart.

        Faceted charts require data to be specified at the top level; if data
        is not specified, the data from the current chart will be used at the
        top level.
        """
        if data is Undefined:
            data = self.data
            self = self.copy()
            self.data = Undefined
        return FacetChart(self, row=row, column=column, data=data, **kwargs)


def layer(*charts, **kwargs):
    """layer multiple charts"""
    return LayerChart(layer=charts, **kwargs)


@utils.use_signature(core.TopLevelFacetSpec)
class FacetChart(TopLevelMixin, core.TopLevelFacetSpec):
    """A Chart with layers within a single panel"""
    def __init__(self, spec, row=Undefined, column=Undefined,
                 facet=Undefined, **kwargs):
        _check_if_valid_subspec(spec, 'FacetChart')
        if facet is Undefined:
            facet = core.FacetMapping()
        if row is not Undefined:
            if isinstance(row, six.string_types):
                row = core.FacetFieldDef(**utils.parse_shorthand(row))
            facet['row'] = row
        if column is not Undefined:
            if isinstance(column, six.string_types):
                column = core.FacetFieldDef(**utils.parse_shorthand(column))
            facet['column'] = column
        if 'data' not in kwargs:
            warnings.warn('FacetChart: data should be defined at the top level')
        super(FacetChart, self).__init__(spec=spec, facet=facet, **kwargs)

    def interactive(self, name=None, bind_x=True, bind_y=True):
        """Make chart axes scales interactive

        Parameters
        ----------
        name : string
            The selection name to use for the axes scales. This name should be
            unique among all selections within the chart.
        bind_x : boolean, default True
            If true, then bind the interactive scales to the x-axis
        bind_y : boolean, default True
            If true, then bind the interactive scales to the y-axis

        Returns
        -------
        chart :
            copy of self, with interactive axes added
        """
        copy = self.copy()
        copy.spec = copy.spec.interactive(name=name, bind_x=bind_x, bind_y=bind_y)
        return copy


def topo_feature(url, feature, **kwargs):
    """A convenience function for extracting features from a topojson url

    Parameters
    ----------
    url : string
        An URL from which to load the data set.

    feature : string
        The name of the TopoJSON object set to convert to a GeoJSON feature collection. For
        example, in a map of the world, there may be an object set named `"countries"`.
        Using the feature property, we can extract this set and generate a GeoJSON feature
        object for each country.

    **kwargs :
        additional keywords passed to TopoDataFormat
    """
    return core.UrlData(url=url,format=core.TopoDataFormat(type='topojson',
                                                         feature=feature, **kwargs))



import sys

PY2 = sys.version_info[0] == 2
PY3 = not PY2

if PY2:
    string_types = basestring,
    integer_types = (int, long)
    import BaseHTTPServer as server
    from StringIO import StringIO as IO
else:
    string_types = str,
    integer_types = int,
    from http import server
    from io import BytesIO as IO



"""
Trellis Histogram
-----------------
This example shows how to make a basic trellis histogram.
https://vega.github.io/vega-lite/examples/trellis_bar_histogram.html
"""
import altair as alt

cars = alt.load_dataset('cars')

chart = alt.Chart(cars).mark_bar().encode(
    alt.X("Horsepower:Q", bin=True),
    y='count(*):Q',
    row='Origin'
)



"""
Main API for Vega-lite spec generation.

DSL mapping Vega types to IPython traitlets.
"""
import six

import pandas as pd

from .schema import core, channels, Undefined

from .data import data_transformers, pipe
from ... import utils
from .display import renderers


SCHEMA_URL = "https://vega.github.io/schema/vega-lite/v1.json"


def _get_channels_mapping():
    mapping = {}
    for attr in dir(channels):
        cls = getattr(channels, attr)
        if isinstance(cls, type) and issubclass(cls, core.SchemaBase):
            mapping[cls] = attr.lower()
    return mapping


#*************************************************************************
# Formula wrapper
# - makes field a required first argument of initialization
# - allows expr trait to be an Expression and processes it properly
#*************************************************************************

class Formula(core.Formula):
    def __init__(self, field, expr=Undefined, **kwargs):
        super(Formula, self).__init__(field=field, expr=expr, **kwargs)


#*************************************************************************
# Top-level Objects
#*************************************************************************

class TopLevelMixin(object):
    _default_spec_values = {"width": 400, "height": 300}
    _class_is_valid_at_instantiation = False

    def _prepare_data(self):
        if isinstance(self.data, (dict, core.Data)):
            pass
        elif isinstance(self.data, pd.DataFrame):
            self.data = pipe(self.data, data_transformers.get())
        elif isinstance(self.data, six.string_types):
            self.data = core.Data(url=self.data)

    def to_dict(self, *args, **kwargs):
        copy = self.copy()
        original_data = getattr(copy, 'data', Undefined)
        copy._prepare_data()

        # We make use of two context markers:
        # - 'data' points to the data that should be referenced for column type
        #   inference.
        # - 'top_level' is a boolean flag that is assumed to be true; if it's
        #   true then a "$schema" arg is added to the dict.
        context = kwargs.get('context', {}).copy()

        is_top_level = context.get('top_level', True)
        context['top_level'] = False

        if original_data is not Undefined:
            context['data'] = original_data
        kwargs['context'] = context

        try:
            dct = super(TopLevelMixin, copy).to_dict(*args, **kwargs)
        except jsonschema.ValidationError:
            dct = None

        # If we hit an error, then re-convert with validate='deep' to get
        # a more useful traceback. We don't do this by default because it's
        # much slower in the case that there are no errors.
        if dct is None:
            kwargs['validate'] = 'deep'
            dct = super(TopLevelMixin, copy).to_dict(*args, **kwargs)

        if is_top_level:
            # since this is top-level we add $schema if it's missing
            if '$schema' not in dct:
                dct['$schema'] = SCHEMA_URL

            # add default values if present
            if copy._default_spec_values:
                dct = utils.update_nested(copy._default_spec_values, dct, copy=True)
        return dct

    def savechart(self, fp, format=None, **kwargs):
        """Save a chart to file in a variety of formats

        Supported formats are json, html, png, svg

        Parameters
        ----------
        fp : string filename or file-like object
            file in which to write the chart.
        format : string (optional)
            the format to write: one of ['json', 'html', 'png', 'eps'].
            If not specified, the format will be determined from the filename.
        **kwargs :
            Additional keyword arguments are passed to the output method
            associated with the specified format.
        """

        if isinstance(fp, six.string_types):
            format = fp.split('.')[-1]

        if format is None:
            raise ValueError("must specify file format: "
                             "['png', 'eps', 'html', 'json']")
        elif format == 'json':
            utils.write_file_or_filename(fp, self.to_json(**kwargs), mode='w')
        elif format == 'html':
            from .html import HTML_TEMPLATE
            opt = dict(renderer=kwargs.pop('renderer', 'canvas'),
                       actions=kwargs.pop('actions', False))
            if opt['renderer'] not in ('canvas', 'svg'):
                raise ValueError("renderer must be 'canvas' or 'svg'")
            spec_html = HTML_TEMPLATE.format(spec=self.to_json(**kwargs),
                                             opt=json.dumps(opt))
            utils.write_file_or_filename(fp, spec_html, mode='w')
        elif format in ['png', 'svg']:
            raise NotImplementedError("saving of v1 specs")
            utils.save_spec(self.to_dict(), fp, format=format, **kwargs)
        else:
            raise ValueError("unrecognized format: '{0}'".format(format))

    # transform method
    @utils.use_signature(core.Transform)
    def transform_data(self, **kwargs):
        """Set the data transform by keyword args."""
        copy = self.copy()
        return utils.update_subtraits(copy, 'transform', **kwargs)

    # Configuration methods
    @utils.use_signature(core.Config)
    def configure(self, **kwargs):
        """Set chart configuration"""
        copy = self.copy()
        return utils.update_subtraits(copy, 'config', **kwargs)

    @utils.use_signature(core.AxisConfig)
    def configure_axis(self, **kwargs):
        """Configure the chart's axes by keyword args."""
        copy = self.copy()
        return utils.update_subtraits(copy, ('config', 'axis'), **kwargs)

    @utils.use_signature(core.CellConfig)
    def configure_cell(self, **kwargs):
        """Configure the chart's cell's by keyword args."""
        copy = self.copy()
        return utils.update_subtraits(copy, ('config', 'cell'), **kwargs)

    @utils.use_signature(core.LegendConfig)
    def configure_legend(self, **kwargs):
        """Configure the chart's legend by keyword args."""
        copy = self.copy()
        return utils.update_subtraits(copy, ('config', 'legend'), **kwargs)

    @utils.use_signature(core.OverlayConfig)
    def configure_overlay(self, **kwargs):
        """Configure the chart's overlay by keyword args."""
        copy = self.copy()
        return utils.update_subtraits(copy, ('config', 'overlay'), **kwargs)

    @utils.use_signature(core.MarkConfig)
    def configure_mark(self, **kwargs):
        """Configure the chart's marks by keyword args."""
        copy = self.copy()
        return utils.update_subtraits(copy, ('config', 'mark'), **kwargs)

    @utils.use_signature(core.ScaleConfig)
    def configure_scale(self, **kwargs):
        """Configure the chart's scales by keyword args."""
        copy = self.copy()
        return utils.update_subtraits(copy, ('config', 'scale'), **kwargs)

    @utils.use_signature(core.FacetConfig)
    def configure_facet(self, **kwargs):
        """Configure the chart's scales by keyword args."""
        copy = self.copy()
        return utils.update_subtraits(copy, ('config', 'facet'), **kwargs)

    @utils.use_signature(core.AxisConfig)
    def configure_facet_axis(self, **kwargs):
        """Configure the facet's axes by keyword args."""
        copy = self.copy()
        return utils.update_subtraits(copy, ('config', 'facet', 'axis'),
                                      **kwargs)

    @utils.use_signature(core.CellConfig)
    def configure_facet_cell(self, **kwargs):
        """Configure the facet's cells by keyword args."""
        copy = self.copy()
        return utils.update_subtraits(copy, ('config', 'facet', 'cell'),
                                      **kwargs)

    @utils.use_signature(core.FacetGridConfig)
    def configure_facet_grid(self, **kwargs):
        """Configure the facet's grid by keyword args."""
        copy = self.copy()
        return utils.update_subtraits(copy, ('config', 'facet', 'grid'),
                                      **kwargs)

    @utils.use_signature(core.FacetScaleConfig)
    def configure_facet_scale(self, **kwargs):
        """Configure the facet's scales by keyword args."""
        copy = self.copy()
        return utils.update_subtraits(copy, ('config', 'facet', 'scale'),
                                      **kwargs)

    # Display related methods

    def _repr_mimebundle_(self, include, exclude):
        """Return a MIME bundle for display in Jupyter frontends."""
        # Catch errors explicitly to get around issues in Jupyter frontend
        # see https://github.com/ipython/ipython/issues/11038
        try:
            dct = self.to_dict()
        except Exception:
            utils.display_traceback(in_ipython=True)
            return {}
        else:
            return renderers.get()(dct)


class Chart(TopLevelMixin, core.ExtendedUnitSpec):
    def __init__(self, data=Undefined, encoding=Undefined, mark=Undefined,
                 width=400, height=300, **kwargs):
        super(Chart, self).__init__(data=data, encoding=encoding, mark=mark,
                                    width=width, height=height, **kwargs)

    @utils.use_signature(core.MarkConfig)
    def mark_area(self, **kwargs):
        """Set the mark to 'area' and optionally specify mark properties"""
        copy = self.copy()
        copy.mark = 'area'
        return copy.configure_mark(**kwargs)

    @utils.use_signature(core.MarkConfig)
    def mark_bar(self, **kwargs):
        """Set the mark to 'bar' and optionally specify mark properties"""
        copy = self.copy()
        copy.mark = 'bar'
        return copy.configure_mark(**kwargs)

    @utils.use_signature(core.MarkConfig)
    def mark_errorBar(self, **kwargs):
        """Set the mark to 'errorBar' and optionally specify mark properties"""
        copy = self.copy()
        copy.mark = 'errorBar'
        return copy.configure_mark(**kwargs)

    @utils.use_signature(core.MarkConfig)
    def mark_line(self, **kwargs):
        """Set the mark to 'line' and optionally specify mark properties"""
        copy = self.copy()
        copy.mark = 'line'
        return copy.configure_mark(**kwargs)

    @utils.use_signature(core.MarkConfig)
    def mark_point(self, **kwargs):
        """Set the mark to 'point' and optionally specify mark properties"""
        copy = self.copy()
        copy.mark = 'point'
        return copy.configure_mark(**kwargs)

    @utils.use_signature(core.MarkConfig)
    def mark_rule(self, **kwargs):
        """Set the mark to 'rule' and optionally specify mark properties"""
        copy = self.copy()
        copy.mark = 'rule'
        return copy.configure_mark(**kwargs)

    @utils.use_signature(core.MarkConfig)
    def mark_text(self, **kwargs):
        """Set the mark to 'text' and optionally specify mark properties"""
        copy = self.copy()
        copy.mark = 'text'
        return copy.configure_mark(**kwargs)

    @utils.use_signature(core.MarkConfig)
    def mark_tick(self, **kwargs):
        """Set the mark to 'tick' and optionally specify mark properties"""
        copy = self.copy()
        copy.mark = 'tick'
        return copy.configure_mark(**kwargs)

    @utils.use_signature(core.MarkConfig)
    def mark_circle(self, **kwargs):
        """Set the mark to 'circle' and optionally specify mark properties"""
        copy = self.copy()
        copy.mark = 'circle'
        return copy.configure_mark(**kwargs)

    @utils.use_signature(core.MarkConfig)
    def mark_square(self, **kwargs):
        """Set the mark to 'square' and optionally specify mark properties"""
        copy = self.copy()
        copy.mark = 'square'
        return copy.configure_mark(**kwargs)

    @utils.use_signature(core.Encoding)
    def encode(self, *args, **kwargs):
        """Define the encoding for the Chart."""
        if args:
            mapping = _get_channels_mapping()
            for arg in args:
                encoding = mapping.get(type(arg), None)
                if encoding is None:
                    raise NotImplementedError("non-keyword arg of type {0}"
                                              "".format(type(arg)))
                if encoding in kwargs:
                    raise ValueError("encode: encoding {0} specified twice"
                                     "".format(encoding))
                kwargs[encoding] = arg

        for prop, field in list(kwargs.items()):
            if not isinstance(field, core.SchemaBase):
                cls = getattr(channels, prop.title())
                # Don't validate now, because field will be computed
                # as part of the to_dict() call.
                kwargs[prop] = cls.from_dict(field, validate=False)
        return utils.update_subtraits(self, 'encoding', **kwargs)



import os
import re


def iter_examples():
    """Iterate over the examples in this directory.

    Each item is a dict with the following keys:
    - "name" : the unique name of the example
    - "filename" : the full file path to the example
    """
    example_dir = os.path.abspath(os.path.dirname(__file__))
    for filename in os.listdir(example_dir):
        name, ext = os.path.splitext(filename)
        if name.startswith('_') or ext != '.py':
            continue
        yield {'name': name,
               'filename': os.path.join(example_dir, filename)}



D3_JS_URL = "https://d3js.org/d3.v3.min.js"
VEGA_JS_URL = "https://cdnjs.cloudflare.com/ajax/libs/vega/2.6.5/vega.min.js"
VEGALITE_JS_URL = "https://cdnjs.cloudflare.com/ajax/libs/vega-lite/1.2.1/vega-lite.min.js"
VEGAEMBED_JS_URL = "https://cdnjs.cloudflare.com/ajax/libs/vega-embed/2.2.0/vega-embed.min.js"



"""
Anscombe’s Quartet
------------------

This example shows how to use the column channel to make a trellis plot. Anscombe's Quartet is a famous dataset constructed by Francis Anscombe. Common summary statistics are identical for each subset of the data, despite the subsets having vastly different characteristics.
"""

import altair as alt
from vega_datasets import data

anscombe = data.anscombe()

chart = alt.Chart(anscombe).mark_circle().encode(
    alt.X('X', scale=alt.Scale(zero=False)),
    alt.Y('Y', scale=alt.Scale(zero=False)), 
    column='Series'
).properties(
    width=150,
    height=150
)



"""
Natural Disasters
-----------------
This example shows a visualization of global deaths from natural disasters.
"""

import altair as alt
from altair.expr import datum
from vega_datasets import data

source = data.disasters.url

chart = alt.Chart(source).mark_circle(
    opacity=0.8,
    stroke='black',
    strokeWidth=1
).encode(
    alt.X('Year:O', axis = alt.Axis(labelAngle = 0)),
    alt.Y('Entity:N'),
    alt.Size('Deaths:Q',
        scale=alt.Scale(range=[0, 5000]),
        legend=alt.Legend(title='Annual Global Deaths')
    ),
    alt.Color('Entity:N', legend=None)
).properties(
    width=480,
    height=350
).transform_filter(
    datum.Entity != 'All natural disasters'
)



"""
Multi-Line Highlight
====================
This multi-line chart uses an invisible Voronoi tessellation to handle mouseover to
identify the nearest point and then highlight the line on which the point falls.
It is adapted from the Vega-Lite example found at
https://bl.ocks.org/amitkaps/fe4238e716db53930b2f1a70d3401701
"""
# category: interactive

import altair as alt
from vega_datasets import data

stocks = data.stocks.url

highlight = alt.selection(type='single', on='mouseover',
                          fields=['symbol'], nearest=True)

base = alt.Chart(stocks).encode(
    x='date:T',
    y='price:Q',
    color='symbol:N'
)

points = base.mark_circle().encode(
    opacity=alt.value(0)
).properties(
    selection=highlight,
    width=600
)

lines = base.mark_line().encode(
    size=alt.condition(~highlight, alt.value(1), alt.value(3))
)

chart = points + lines
